package org.rcisoft.business.buildtp.service.impl;



import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.manage.dao.ProjectRepository;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.business.buildtp.dao.BuildTpRepository;
import org.rcisoft.business.buildtp.entity.BuildTp;
import org.rcisoft.business.buildtp.service.BuildTpService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-4-3 10:11:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BuildTpServiceImpl implements BuildTpService {

    @Autowired
    private BuildTpRepository buildTpRepository;
    @Autowired
    private ProjectRepository projectRepository;


    /**
    * 保存 buildTp
    * @param buildTp
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BuildTp buildTp){
        int line = 0;
        // 验重
        Integer repeat = this.repeat(null, buildTp.getBldTpNm());
        if (repeat > 0)
            return new PersistModel(line, "建筑类型不能重复");
        buildTp.setBldTpId(UUID.randomUUID().toString().replace("-",""));
        //增加操作
        line = buildTpRepository.insertSelective(buildTp);
        return new PersistModel(line);
    }

    /**
     * 删除建筑类型
     * @param buildTp
     * @return
     */
    @Override
    public PersistModel delete(BuildTp buildTp) {
        int line = 0;
        String message = "";
        Example example = new Example(BuildTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("bldTpId", buildTp.getBldTpId());
        if(buildTp.getBldTpId()!=null && !buildTp.getBldTpId().equals("")){
            line = buildTpRepository.deleteByExample(example);
        }else{
            message = "ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }


    /**
    * 修改 buildTp
    * @param buildTp
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BuildTp buildTp){
        int line = 0;
        // 验重
        Integer repeat = this.repeat(buildTp.getBldTpId(), buildTp.getBldTpNm());
        if (repeat > 0)
            return new PersistModel(line, "建筑类型不能重复");
        Example example = new Example(BuildTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("bldTpId", buildTp.getBldTpId());
        line = buildTpRepository.updateByExampleSelective(buildTp,example);
        return new PersistModel(line);
    }


    /**
    * 设备类型列表，不需要分页
    * @return
    */
    @Override
    public List<BuildTp> queryBuildTpList(){
        return buildTpRepository.queryBuildTps();
    }

    /**
     * 验重
     * @param bldTpNm
     * @return
     */
    private Integer repeat(String bldTpId, String bldTpNm) {
        Example example = new Example(BuildTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("bldTpNm", bldTpNm);
        if (StringUtils.isNotEmpty(bldTpId))
            criteria.andNotEqualTo("bldTpId", bldTpId);
        return buildTpRepository.selectCountByExample(example);
    }

}
