package org.rcisoft.business.device.assets.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import com.google.zxing.WriterException;
import org.apache.poi.ss.util.CellRangeAddress;
import org.rcisoft.business.device.assets.dao.BusDeviceRepository;
import org.rcisoft.business.device.assets.service.BusDeviceService;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.business.device.assets.vo.DeviceParamsVo;
import org.rcisoft.business.mainte.paramlibrary.dao.ParamLibraryRepository;
import org.rcisoft.business.mainte.paramlibrary.entity.BusParamLibrary;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.business.manage.entity.BusDevicetpParam;
import org.rcisoft.business.manage.entity.BusLibraryParam;
import org.rcisoft.business.manage.service.BusDeviceParamService;
import org.rcisoft.business.manage.service.BusDevicetpParamService;
import org.rcisoft.business.manage.service.BusLibraryParamService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.device.assets.entity.BusDevice;

import org.rcisoft.core.util.QRCodeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by  on 2018-4-12 15:33:23.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusDeviceServiceImpl implements BusDeviceService {

    @Autowired
    private BusDeviceRepository busDeviceRepository;
    @Autowired
    private BusDeviceParamService busDeviceParamService;
    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;
    @Autowired
    private BusDevicetpParamService busDevicetpParamService;
    @Autowired
    private ParamLibraryRepository paramLibraryRepository;

    /**
     * 二维码存放路径
     */
    @Value("${filepath.qrcode}")
    String qrcodePath;

    /**
    * 保存 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusDevice busDevice){
        busDevice.setCreateTime(new Date());
        int line = 1;
        String message = "";
        //增加操作
        busDevice.setDevId(UUID.randomUUID().toString().replace("-",""));
        boolean flag = busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum())!=null && busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum()).size()>0;
        if(flag){
            line = 0;
            message = "设备编号已存在，新增失败";
        }else{
            // 保存二维码路径（配置的路径+项目编号）
            String path = qrcodePath + busDevice.getProId() + "/";
            try {
                // 生成二维码(第一个参数（二维码数据）：设备编号，第二个参数（保存的路径）：保存的路径，第三个参数（文件名）：设备id)
                QRCodeUtils.createQRCodeFile(busDevice.getDevNum(), path, busDevice.getDevId());
            } catch (WriterException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            // 将二维码路径设置到对象中（项目id/设备id.JPG）
            busDevice.setQrcodeUrl(busDevice.getProId() + "/" + busDevice.getDevId() + ".JPG");
            line = busDeviceRepository.insertSelective(busDevice);
        }

        return new PersistModel(line, message);
    }


    /**
    * 修改 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusDevice busDevice){
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",busDevice.getDevId());
        if(busDevice.getDevId()!=null && !busDevice.getDevId().equals("")){
            line = busDeviceRepository.updateByExampleSelective(busDevice,example);
        }else{
            line = 0;
            message = "设备ID为空，更新失败";
        }

        return new PersistModel(line,message);
    }

    /**
     * 删除
     * @param devid
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devid) {
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",devid);
        Map<String,Object> map = new HashMap<>();
        map.put("dev_id",devid);
        if(devid != null && !devid.equals("")){
            if(busDeviceRepository.queryOriginalByDev(map)==null || busDeviceRepository.queryOriginalByDev(map).size()<1){
                line = busDeviceRepository.deleteByExample(example);
            }else{
                message = "设备存在关联数据，删除失败";
            }
        }else{
            message = "设备ID为空，删除失败";
        }

        return new PersistModel(line,message);
    }

    /**
    * 根据id查询 busDevice
    * @param id
    * @return
    */
    @Override
    public List<BusDevice> findById(String id){
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",id);
        return busDeviceRepository.selectByExample(example);
    }

    @Override
    public List<Map<String,Object>> findByDevNum(String devNum){
        BusDevice busDevice = new BusDevice();
        busDevice.setDevNum(devNum);
        return busDeviceRepository.queryBusDevices(busDevice);
    }

    /**
    * 分页查询 busDevice
    * @param busDevice
    * @return
    */
    @Override
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusDevice> paginationUtility,
                                               BusDevice busDevice){
        //避免查出所有设备
        if(busDevice.getProId()!=null && !busDevice.getProId().equals("")){
            List<Map<String,Object>> list = busDeviceRepository.queryBusDevices(busDevice);
            for(Map<String,Object> tmap : list){
                tmap.put("paramList",busDeviceParamService.findByDev(tmap.get("DEV_NUM").toString(),tmap.get("PRO_ID").toString()));
                tmap.put("libraryparam",busLibraryParamRepository.queryLibraryParamByDev(tmap.get("DEV_NUM").toString()));
                BusDevicetpParam busDevicetpParam = new BusDevicetpParam();
                busDevicetpParam.setProId(tmap.get("PRO_ID").toString());
                busDevicetpParam.setDevTpId(tmap.get("DEV_TP_ID").toString());
                tmap.put("deviceTpParam",busDevicetpParamService.queryBusDevicetpParam(busDevicetpParam));
            }
            return list;
        }
        return null;
    }
    /*
        通过项目id与子系统id获取子系统中的所有设备
        @param sysId  子系统id
        @param proId  项目id
         */
    @Override
    public List<DeviceAssetStatisticVo> getDevicesBySysId(String sysId, String proId) {
        return busDeviceRepository.getBusDevicesBySysId(sysId,proId);
    }

    @Override
    public String getProNameByProId(String proId) {
        return busDeviceRepository.getProNameByProId(proId);
    }

    @Override
    public String getSysNameBySysId(String sysId) {
        return busDeviceRepository.getSysNameBySysId(sysId);
    }

    @Transactional
    @Override
    public Integer addRunTime() {
        //最后需要保存的数据
        List<BusDevice> saveList = new ArrayList<>();
        //得到当前时间前10分钟的时间
        String time = new SimpleDateFormat("yyyy-MM-dd HH:mm").format(new Date(System.currentTimeMillis() - 900000));
        //查询10分钟前所有数据的耗电量信息
        List<Map<String, Object>> list = busDeviceRepository.selectRunTime(time);
        list.forEach(map -> {
            BigDecimal shutPower = (BigDecimal) map.get("SHUT_POWER");
            shutPower = shutPower == null ? new BigDecimal(0) : shutPower;
            BigDecimal elec = (BigDecimal) map.get("ELEC");
            elec = elec == null ? new BigDecimal(0) : elec;
            //实际电量 > 关闭电量
            if (elec.compareTo(shutPower) == 1) {
                BusDevice busDevice = new BusDevice();
                busDevice.setDevId((String) map.get("DEV_ID"));
                BigDecimal runTm = (BigDecimal) map.get("RUN_TM");
                runTm = runTm == null ? new BigDecimal(0) : runTm;
                runTm = runTm.add(new BigDecimal(600));
                busDevice.setRunTm(runTm);
                saveList.add(busDevice);
            }
        });
        //更新结果
        if (saveList.size() > 0)
            return busDeviceRepository.batchUpdateRunTm(saveList);
        else
            return 0;
    }

    @Override
    public void downloadExcel(HttpServletResponse response, String devNums) {
        try (OutputStream outputStream = response.getOutputStream()) {
            response.setHeader("Content-disposition", "attachment;filename=download.xls");
            outputStream.write(this.createExcel(devNums));
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public PersistModel generateCode(String devNum) {
        int line = 1;
        String message = "";
        boolean flag = busDeviceRepository.queryDeviceByDevNum(devNum)!=null && busDeviceRepository.queryDeviceByDevNum(devNum).size()>0;
        if(!flag){
            line = 0;
            message = "设备编号不存在，生成失败";
        }else{
            // 保存二维码路径（配置的路径+项目编号）
            String path = qrcodePath + "/";
            try {
                // 生成二维码(第一个参数（二维码数据）：设备编号，第二个参数（保存的路径）：保存的路径，第三个参数（文件名）：设备id)
                QRCodeUtils.createQRCodeFile(devNum, path,devNum);
            } catch (WriterException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            Example example = new Example(BusDevice.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("devNum",devNum);
            BusDevice busDevice = new BusDevice();
            busDevice.setDevNum(devNum);
            busDevice.setQrcodeUrl(devNum + ".JPG");
            line = busDeviceRepository.updateByExampleSelective(busDevice,example);
        }
        return new PersistModel(line,message);
    }

    @Override
    public Integer update(String devNum, Integer errorNum) {
        BusDevice busDevice = new BusDevice();
        busDevice.setDevNum(devNum);
        busDevice.setErrorNum(errorNum);
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devNum", devNum);
        return busDeviceRepository.updateByExampleSelective(busDevice, example);
    }

    private byte[] createExcel(String devNums) throws IOException {
        int year = Calendar.getInstance().get(Calendar.YEAR);
        HSSFWorkbook workbook = null;
        try(ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            workbook = new HSSFWorkbook();
            // 拆分devNums
            String[] numArray = devNums.split(",");
            for (String devNum : numArray) {
                int rowNum = 0;
                //定义sheet
                HSSFSheet sheet = workbook.createSheet(devNum);
                //第一行：设备编号 + 年份
                HSSFRow row_1 = sheet.createRow(rowNum++);
                row_1.createCell(0, CellType.STRING).setCellValue("设备编号");
                row_1.createCell(1, CellType.STRING).setCellValue(devNum);
                row_1.createCell(3, CellType.STRING).setCellValue("年份");
                row_1.createCell(4, CellType.STRING).setCellValue(Calendar.getInstance().get(Calendar.YEAR));
                //空一行
                rowNum++;
                // 第三行：参数、参数名称、参数编码
                String[] title_3 = new String[]{"参数", "名称", "编码", "编码code", "来源", "类型"};
                HSSFRow row_3 = sheet.createRow(rowNum++);
                for (int i = 0; i < title_3.length; i++) {
                    row_3.createCell(i, CellType.STRING).setCellValue(title_3[i]);
                }
                //循环参数列表
                List<Map<String, String>> paramsList = busLibraryParamRepository.selectParamsByDevNum(devNum);
                int size = paramsList.size();
                for (int i = 0; i < size; i++) {
                    Map<String, String> param = paramsList.get(i);
                    HSSFRow row = sheet.createRow(rowNum++);
                    row.createCell(0, CellType.STRING).setCellValue(i + 1);//编号
                    row.createCell(1, CellType.STRING).setCellValue(param.get("paramNm"));//参数名称
                    row.createCell(2, CellType.STRING).setCellValue(param.get("param"));//参数编码
                    row.createCell(3, CellType.STRING).setCellValue(param.get("paramCode"));//编码code
                    row.createCell(4, CellType.STRING).setCellValue(param.get("source").equals("1") ? "设备" : "传感器");//来源
                    // 给主参数加上注释
                    if (StringUtils.equals(param.get("compareFlag"), "1"))
                        row.createCell(5, CellType.STRING).setCellValue("主参数");
                }
                // 表明参数类型行
                rowNum = rowNum + 4 - size;
                // 合并单元格
                CellRangeAddress cra1 = new CellRangeAddress(7, 7, 3, 4);
                CellRangeAddress cra2 = new CellRangeAddress(7, 7, 5, 6);
                //在sheet里增加合并单元格
                sheet.addMergedRegion(cra1);
                sheet.addMergedRegion(cra2);
                HSSFRow row_param_title = sheet.createRow(rowNum);
                row_param_title.createCell(3, CellType.STRING).setCellValue("主参数");
                row_param_title.createCell(5, CellType.STRING).setCellValue("其他参数");
                rowNum++;
                //添加参数值表头列
                HSSFRow row_next = sheet.createRow(rowNum);
                row_next.createCell(0, CellType.STRING).setCellValue("参数值列表");
                row_next.createCell(1, CellType.STRING).setCellValue("功率（kw）");
                row_next.createCell(2, CellType.STRING).setCellValue("用气速率（m3/h）");
                for (int i = 0; i < 4; i++) {
                    row_next.createCell(3 + i, CellType.STRING).setCellValue("参数" + (i + 1) + "值");
                }
                //查询参数库列表
                Example example = new Example(BusParamLibrary.class);
                Example.Criteria criteria = example.createCriteria();
                criteria.andEqualTo("devNum", devNum);
                criteria.andEqualTo("year", year);
                List<BusParamLibrary> list = paramLibraryRepository.selectByExample(example);
                //循环放入当前sheet
                rowNum++;
                for (BusParamLibrary busParamLibrary : list) {
                    HSSFRow row = sheet.createRow(rowNum++);
                    row.createCell(1, CellType.STRING).setCellValue(busParamLibrary.getPower() == null ? "" : busParamLibrary.getPower().toString());
                    row.createCell(2, CellType.STRING).setCellValue(busParamLibrary.getGasSpeed() == null ? "" : busParamLibrary.getGasSpeed().toString());
                    row.createCell(3, CellType.STRING).setCellValue(busParamLibrary.getP1V() == null ? "" : busParamLibrary.getP1V().toString());
                    row.createCell(4, CellType.STRING).setCellValue(busParamLibrary.getP2V() == null ? "" : busParamLibrary.getP2V().toString());
                    row.createCell(5, CellType.STRING).setCellValue(busParamLibrary.getP3V() == null ? "" : busParamLibrary.getP3V().toString());
                    row.createCell(6, CellType.STRING).setCellValue(busParamLibrary.getP3V() == null ? "" : busParamLibrary.getP4V().toString());
                }
            }
            workbook.write(os);
            return os.toByteArray();
        } finally {
            workbook.close();
        }
    }

}
