package org.rcisoft.business.device.report.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.device.report.service.FormulaService;
import org.rcisoft.business.device.report.vo.FormulaVo;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by JiChao on 2018/8/29.
 * 公式
 */
@Api(tags = "设备报表--公式")
@RestController
@RequestMapping("formula")
public class FormulaController {

    @Autowired
    private FormulaService formulaServiceImpl;

    @ApiOperation(value="新增公式和变量", notes="新增公式和变量")
    @RequestMapping("/addFormulaVariable")
    public Result addFormulaVariable(@RequestBody FormulaVo formulaVo) {
        Integer result = formulaServiceImpl.addFormulaVariable(formulaVo);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="修改公式和变量", notes="修改公式和变量")
    @RequestMapping("/updateFormulaVariable")
    public Result updateFormulaVariable(@RequestBody FormulaVo formulaVo) {
        Integer result = formulaServiceImpl.updateFormulaVariable(formulaVo);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="删除公式和变量", notes="删除公式和变量")
    @RequestMapping("/deleteFormulaVariable")
    public Result deleteFormulaVariable(@RequestParam String formulaId) {
        Integer result = formulaServiceImpl.deleteFormulaVariable(formulaId);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="查询公式和变量", notes="查询公式和变量")
    @RequestMapping("/selectFormulaVariable")
    public Result selectFormulaVariable(@RequestParam String formulaId) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, formulaServiceImpl.selectFormulaVariable(formulaId));
    }

    @ApiOperation(value="查询公式列表", notes="查询公式列表")
    @RequestMapping("/selectFormulaList")
    public Result selectFormulaList(@RequestParam String proId) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, formulaServiceImpl.selectFormulaList(proId));
    }

    @ApiOperation(value="导出公式", notes="导出公式")
    @RequestMapping("/downloadFormula")
    public void downloadFormula(HttpServletResponse response, String formulaId, String start, String end) {
        formulaServiceImpl.downloadFormula(response, formulaId, start, end);
    }

    @ApiOperation(value="批量导出公式", notes="批量导出公式")
    @RequestMapping("/downloadFormulas")
    public void downloadFormulas(HttpServletResponse response, String proId, String formulaIds, String start, String end) {
        formulaServiceImpl.downloadFormulas(response, proId, formulaIds, start, end);
    }
}
