package org.rcisoft.business.device.report.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.greenpineyu.fel.FelEngine;
import com.greenpineyu.fel.FelEngineImpl;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.util.CellRangeAddress;
import org.rcisoft.business.device.report.dao.FormulaRepository;
import org.rcisoft.business.device.report.dao.TotalDataRepository;
import org.rcisoft.business.device.report.entity.Formula;
import org.rcisoft.business.device.report.entity.TotalData;
import org.rcisoft.business.device.report.entity.Variable;
import org.rcisoft.business.device.report.service.FormulaService;
import org.rcisoft.business.device.report.service.VariableService;
import org.rcisoft.business.device.report.vo.FormulaVo;
import org.rcisoft.core.util.UuidUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * Created by JiChao on 2018/8/31.
 */
@Service
public class FormulaServiceImpl implements FormulaService {

    @Autowired
    private FormulaRepository formulaRepository;
    @Autowired
    private VariableService variableServiceImpl;
    @Autowired
    private TotalDataRepository totalDataRepository;


    @Override
    public Integer addFormula(Formula formula) {
        formula.setCreateTime(new Date());
        return formulaRepository.addFormula(formula);
    }

    @Override
    public Integer updateFormula(Formula formula) {
        return formulaRepository.updateFormula(formula);
    }

    @Override
    public Integer deleteFormula(String id) {
        return formulaRepository.deleteFormula(id);
    }

    @Override
    public Formula selectFormula(String id) {
        return formulaRepository.selectFormula(id);
    }

    @Override
    public List<Formula> selectFormulaList(String proId) {
        return formulaRepository.selectFormulaList(proId);
    }

    @Transactional
    @Override
    public Integer addFormulaVariable(FormulaVo formulaVo) {
        Formula formula = formulaVo.getFormula();
        String uuid = UuidUtil.create32();
        formula.setId(uuid);
        Integer i = this.addFormula(formula);
        Integer k = variableServiceImpl.addVariable(formulaVo.getVariableList(), uuid, formula.getProId());
        return i;
    }

    @Transactional
    @Override
    public Integer updateFormulaVariable(FormulaVo formulaVo) {
        Formula formula = formulaVo.getFormula();
        String id = formula.getId();
        Integer i = this.updateFormula(formula);
        Integer j = variableServiceImpl.deleteVariable(id);
        Integer k = variableServiceImpl.addVariable(formulaVo.getVariableList(), id, formula.getProId());
        return i;
    }

    @Transactional
    @Override
    public Integer deleteFormulaVariable(String formulaId) {
        Integer i = this.deleteFormula(formulaId);
        Integer j = variableServiceImpl.deleteVariable(formulaId);
        return i;
    }

    @Transactional
    @Override
    public FormulaVo selectFormulaVariable(String formulaId) {
        Formula formula = this.selectFormula(formulaId);
        List<Variable> variableList = variableServiceImpl.selectVariable(formulaId);
        return new FormulaVo(formula, variableList);
    }

    @Override
    public void downloadFormula(HttpServletResponse response, String formulaId, String start, String end) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 公式计算
        FelEngine fel = new FelEngineImpl();
        // 根据id查找公式
        Formula formula = this.selectFormula(formulaId);
        // 根据id查找变量
        List<Variable> variableList = variableServiceImpl.selectVariable(formulaId);
        // 根据proId和时间区间，查找数据list
        String proId = formula.getProId();
        start = start + " 00:00:00";
        end = end + " 23:59:59";
        List<TotalData> totalDataList = totalDataRepository.selectTotalData(proId, start, end);
        // 生成excel
        HSSFWorkbook workbook = new HSSFWorkbook();
        //定义sheet
        HSSFSheet sheet = workbook.createSheet("sheet1");
        // 创建第一行
        int i = 0;
        HSSFRow head = sheet.createRow(i++);
        head.createCell(0, CellType.STRING).setCellValue("时间");
        head.createCell(1, CellType.STRING).setCellValue("数值");
        // 循环数据list
        for (TotalData totalData : totalDataList) {
            // 创建row
            HSSFRow row = sheet.createRow(i++);
            // 公式
            String f = formula.getFormula();
            // 得到原始数据
            String json = totalData.getJson();
            // 变成json对象
            JSONObject jsonObject = JSON.parseObject(json);
            // 循环变量list
            for (Variable variable : variableList) {
                f = this.replace(f, variable, jsonObject);
            }
            String eval = this.getResult(f, fel);
            // 放入row中
            row.createCell(0, CellType.STRING).setCellValue(sdf.format(totalData.getTm()));
            row.createCell(1, CellType.STRING).setCellValue(eval);
        }
        //下载
        this.download(response, workbook, formula.getName());
    }

    @Override
    public void downloadFormulas(HttpServletResponse response, String proId, String formulaIds, String start, String end) {
        String[] ids = formulaIds.split(",");
        // 查询时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        start = start + " 00:00:00";
        end = end + " 23:59:59";
        // 公式计算类
        FelEngine fel = new FelEngineImpl();
        // 根据ids查找公式列表
        List<Formula> formulaList = formulaRepository.selectFormulas(ids);
        // 根据id查找变量
        List<Variable> variableList = variableServiceImpl.selectVariables(ids);
        // 根据proId和时间区间，查找数据list
        List<TotalData> totalDataList = totalDataRepository.selectTotalData(proId, start, end);
        // 生成excel
        HSSFWorkbook workbook = new HSSFWorkbook();
        //定义sheet
        HSSFSheet sheet = workbook.createSheet("sheet1");
        // 放入标题
        HSSFRow row0 = sheet.createRow(0);
        row0.createCell(0, CellType.STRING).setCellValue("公式名称");
        HSSFRow row1 = sheet.createRow(1);
        row1.createCell(0, CellType.STRING).setCellValue("公式内容");
        // 放入公式
        for (int i = 0; i < formulaList.size(); i++) {
            Formula formula = formulaList.get(i);
            row0.createCell(i + 1, CellType.STRING).setCellValue(formula.getName());
            row1.createCell(i + 1, CellType.STRING).setCellValue(formula.getFormula());
        }
        HSSFRow row2 = sheet.createRow(2);
        row2.createCell(0, CellType.STRING).setCellValue("时间");
        CellRangeAddress cra=new CellRangeAddress(2, 2, 1, formulaList.size());// 合并单元格
        sheet.addMergedRegion(cra);
        //合并的单元格样式
        HSSFCellStyle boderStyle = workbook.createCellStyle();
        //水平居中
        boderStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        HSSFCell cell = row2.createCell(1, CellType.STRING);
        cell.setCellStyle(boderStyle);
        cell.setCellValue("数值");
//        row2.createCell(1, CellType.STRING).setCellValue("数值");
        // 循环数据list
        for (int i = 0; i < totalDataList.size(); i++) {
            TotalData totalData = totalDataList.get(i);
            // 得到json对象
            JSONObject jsonObject = JSON.parseObject(totalData.getJson());
            // 创建row
            HSSFRow row = sheet.createRow(i + 3);
            // 放入时间
            row.createCell(0, CellType.STRING).setCellValue(sdf.format(totalData.getTm()));
            // 循环公式
            for (int j = 0; j < formulaList.size(); j++) {
                Formula formula = formulaList.get(j);
                String formulaId = formula.getId();
                // 公式
                String formulaFormula = formula.getFormula();
                // 循环变量list
                for (int k = 0; k < variableList.size(); k++) {
                    Variable variable = variableList.get(k);
                    // 找到属于这个公式的参数
                    if (StringUtils.equals(variable.getFormulaId(), formulaId)) {
                        formulaFormula = this.replace(formulaFormula, variable, jsonObject);
                    }
                }
                // 计算出公式的最终结果
                String eval = this.getResult(formulaFormula, fel);
                row.createCell(j + 1, CellType.STRING).setCellValue(eval);
            }
        }
        //下载
        this.download(response, workbook, "formula");
    }

    private String replace(String formula, Variable variable, JSONObject jsonObject) {
        // 公式中的变量
        String v = variable.getVariable();
        // 来源
        String s = variable.getSource();
        // 参数
        String p = variable.getParam();
        // 根据来源参数，获取json中对应的值
        String value = null;
        JSONObject s_json = (JSONObject) jsonObject.get(s);
        if (s_json != null) {
            JSONObject r_json = (JSONObject) s_json.get("REG_VAL");
            if (r_json != null) {
                Object p_json = r_json.get(p);
                if (p_json != null) {
                    value = p_json.toString();
                }
            }
        }
        if (value != null)
            // 替换公式
            formula = formula.replaceAll(v, value);
        return formula;
    }

    private String getResult(String formula, FelEngine fel) {
        // 结果
        String eval = "";
        // 判断表达式是否由数字和运算符号组成
        boolean flag = true;
        String[] split = formula.split("");
        for (String s : split) {
            if (!s.matches("\\d|\\(|\\)|\\+|\\-|\\*|/|%|\\.")) {
                flag = false;
                break;
            }
        }
        // 计算出公式的最终结果
        if (flag) {
            Object result = fel.eval(formula);
            if (result != null) {
                eval = result.toString();
            }
        }
        return eval;
    }

    private void download(HttpServletResponse response, HSSFWorkbook workbook, String formulaName) {
        //下载
        try(OutputStream outputStream = response.getOutputStream()) {
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + formulaName + ".xls");
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
