package org.rcisoft.business.device.report.service.impl;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.business.device.report.dao.ReportRepository;
import org.rcisoft.business.device.report.entity.Original;
import org.rcisoft.business.device.report.entity.Report;
import org.rcisoft.business.device.report.service.ReportService;
import org.rcisoft.business.device.report.vo.DeviceVo;
import org.rcisoft.business.device.report.vo.Params;
import org.rcisoft.business.device.report.vo.ReportVo;
import org.rcisoft.business.mainte.adaptive.dao.BusTemperatureRepository;
import org.rcisoft.business.mainte.adaptive.entity.BusTemperature;
import org.rcisoft.business.manage.dao.SysCityRepository;
import org.rcisoft.business.manage.dao.SysProvinceRepository;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.entity.SysCity;
import org.rcisoft.business.manage.entity.SysProvince;
import org.rcisoft.business.manage.service.BusProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by JiChao on 2018/5/3.
 */
@Service
public class ReportServiceImpl implements ReportService {

    @Value("${filepath.equipment}")
    private String filePath;

    @Autowired
    private ReportRepository reportRepository;
    @Autowired
    private BusProjectService busProjectServiceImpl;
    @Autowired
    private SysProvinceRepository sysProvinceRepository;
    @Autowired
    private SysCityRepository sysCityRepository;
    @Autowired
    private BusTemperatureRepository busTemperatureRepository;

    @Override
    public void download(HttpServletResponse response, String proId, Integer year, Integer month, Integer day) {
        byte[] bytes = null;
        //如果有文件
        String fileName = filePath + proId + File.separator + year + File.separator + month + File.separator + day + ".xls";
        File file = new File(fileName);
        if (!file.getParentFile().exists()) file.getParentFile().mkdirs();
        try(OutputStream outputStream = response.getOutputStream()) {
            // 如果已经存在文件，直接下载
//            if (!file.exists()) {
                // 不存在直接生成一个文件，生成文件太耗时间
            // 总会出问题，还是每次都生成吧
                file = this.createExcel(proId, year, month, day, file);
//            }
            //转成字节
            bytes = FileUtils.readFileToByteArray(file);
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(day + ".xls", "iso-8859-1"));
            outputStream.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void downloadTemperature(HttpServletResponse response, String proId, String start, String end) {
        try(OutputStream outputStream = response.getOutputStream()) {
            byte[] bytes = this.createTemperature(proId, start, end);
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=temperature.xls");
            outputStream.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

//    @Override
//    public List<Integer> getExistFiles(String proId, Integer year, Integer month) {
//        // 返回值
//        List<Integer> resultList = new ArrayList<>();
//        // 文件夹名称
//        String dirName = filePath + proId + File.separator + year + File.separator + month;
//        File file = new File(dirName);
//        if (file.exists()) {
//            String[] names = file.list();
//            for (String name : names) {
//                resultList.add(Integer.parseInt(StringUtils.split(name, ".")[0]));
//            }
//        }
//        return resultList;
//    }

    /**
     * 查找，组合数据，生成文件
     * @param proId
     * @param year
     * @param month
     * @param day
     * @param file
     * @return
     */
    private File createExcel(String proId, Integer year, Integer month, Integer day, File file) {
        //查询条件
        Params p = new Params(proId, year, month, day, year + "-" + month + "-" + day);
        //得到设备的结果集
        List<ReportVo> reportVoList = this.getReportVoList(p);
        //得到数据的结果集
        List<Original> originalList = reportRepository.originalList(p);
        //---------------定义excel-----------------------
        HSSFWorkbook workbook = new HSSFWorkbook();
        //---------------定义excel-----------------------
        if (reportVoList.size() > 0) {
            //循环设备结果集，每个设备创建一个sheet页
            reportVoList.forEach(reportVo -> {
                //定义sheet
                HSSFSheet sheet = workbook.createSheet(reportVo.getDevNm());
                //创建表头
                HSSFRow head = sheet.createRow(0);
                List<DeviceVo> deviceList = reportVo.getDeviceList();
                //第一列填时间
                head.createCell(0, CellType.STRING).setCellValue("时间");
                int size = deviceList.size();
                for (int i = 0; i < size; i++) {
                    HSSFCell cell = head.createCell(i + 1, CellType.STRING);
                    //表头填入参数名称
                    cell.setCellValue(deviceList.get(i).getParamNm());
                }
                //加入两列，用电量，用气量
//                head.createCell(size + 1, CellType.STRING).setCellValue("用电量");
//                head.createCell(size + 2, CellType.STRING).setCellValue("用气量");
                //定义一个增量
                int x = 1;
                //循环数据
                for (int i = 0; i < originalList.size(); i++) {
                    Original original = originalList.get(i);
                    String devNum = original.getDevNum();
                    //判断devNum是否等于当前设备的devNum
                    if (StringUtils.equals(devNum, reportVo.getDevNum())) {
                        //第一列放入时间
                        HSSFRow row = sheet.createRow(x++);
                        row.createCell(0, CellType.STRING).setCellValue(original.getTm());
                        //设备的参数对象
                        JSONObject originalJson = JSONObject.parseObject(original.getOriginalStr());
                        //传感器参数对象
                        JSONObject sensorJson = JSONObject.parseObject(original.getSensorJson());
                        //循环设备
                        for (int j = 0; j < size; j++) {
                            DeviceVo deviceVo = deviceList.get(j);
                            String param = deviceVo.getParam();//参数code
                            String psource = deviceVo.getPsource();//参数来源，1：设备 2：传感器
                            //需要保存的结果
                            String res = "";
                            if (psource.equals("1")) res = originalJson.get(param) == null ? "" : originalJson.get(param).toString();
                            else if (psource.equals("2")) res = sensorJson.get(param) == null ? "" : sensorJson.get(param).toString();
                            //依次放入行中
                            row.createCell(j + 1, CellType.STRING).setCellValue(res);
                        }
                        //放入表具数据
//                        row.createCell(size + 1, CellType.STRING).setCellValue(original.getElec());
//                        row.createCell(size + 2, CellType.STRING).setCellValue(original.getGas());
                        //用完删除当前行的数据
//                    originalList.remove(i--);
                    }
                }
            });
        } else {
            // 没有数据时，避免报错，添加提示
            HSSFSheet sheet = workbook.createSheet("sheet1");
            HSSFRow head = sheet.createRow(0);
            head.createCell(0, CellType.STRING).setCellValue("没有数据");
        }

        try {
            workbook.write(file);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file;
    }

    /**
     * 转换结果集，变成设备的list，每个设备有自己的参数list
     * list套list形式
     * @param params proId
     * @return
     */
    private List<ReportVo> getReportVoList(Params params) {
        //所有设备参数的list
        List<Report> reportList = reportRepository.reportList(params);
        String _devNum = "";
        //返回的结果
        List<ReportVo> reportVoList = new ArrayList<>();
        //记录最后结果集的标签
        int num = -1;
        for (int i = 0; i < reportList.size(); i++) {
            Report report = reportList.get(i);
            String devNum = report.getDevNum();
            //不等于
            if (!StringUtils.equals(_devNum, devNum)){
                _devNum = devNum;
                ReportVo reportVo = new ReportVo();
                reportVo.setDevNm(report.getDevNm());
                reportVo.setDevNum(devNum);
                //加入新的集合中
                reportVoList.add(reportVo);
                num++;
            }
            //从集合里取到需要添加的记录，将当前的值放入
            ReportVo reportVo = reportVoList.get(num);
            List<DeviceVo> deviceVoList = reportVo.getDeviceList();
            DeviceVo deviceVo = new DeviceVo(report.getParam(), report.getPsource(), report.getParamNm());
            deviceVoList.add(deviceVo);
        }
        return reportVoList;
    }

    /**
     * 生成excel
     * @param proId
     * @param start
     * @param end
     * @return
     */
    private byte[] createTemperature(String proId, String start, String end) throws IOException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        start = start + " 00:00:00";
        end = end + " 23:59:59";
        // 查询项目的信息
        BusProject busProject = busProjectServiceImpl.findByProId(proId);
        String code = busProject.getCode();
        // 查询省份
        String provinceId = busProject.getProvince();
        SysProvince sysProvince = sysProvinceRepository.selectByPrimaryKey(provinceId);
        String provinceName = sysProvince.getName();
        // 查询城市
        String cityId = busProject.getCity();
        SysCity sysCity = sysCityRepository.selectByPrimaryKey(Integer.parseInt(cityId));
        String cityName = sysCity.getName();
        // 根据code、时间，查询温度
        Example example = new Example(BusTemperature.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("code", code);
        criteria.andBetween("tm", start, end);
        List<BusTemperature> list = busTemperatureRepository.selectByExample(example);
        // 生成excel
        HSSFWorkbook workbook = new HSSFWorkbook();
        //定义sheet
        HSSFSheet sheet = workbook.createSheet("sheet1");
        // 创建第一行
        HSSFRow head = sheet.createRow(0);
        // 省份，城市
        head.createCell(0, CellType.STRING).setCellValue("省份");
        head.createCell(1, CellType.STRING).setCellValue(provinceName);
        head.createCell(2, CellType.STRING).setCellValue("城市");
        head.createCell(3, CellType.STRING).setCellValue(cityName);
        // 第二行添加表头
        HSSFRow title = sheet.createRow(1);
        title.createCell(0, CellType.STRING).setCellValue("时间");
        title.createCell(1, CellType.STRING).setCellValue("温度");
        title.createCell(2, CellType.STRING).setCellValue("湿度");
        title.createCell(3, CellType.STRING).setCellValue("风向风速");
        // 填入数据
        for (int i = 0; i < list.size(); i++) {
            HSSFRow row = sheet.createRow(i + 2);
            BusTemperature busTemperature = list.get(i);
            row.createCell(0, CellType.STRING).setCellValue(sdf.format(busTemperature.getTm()));
            row.createCell(1, CellType.STRING).setCellValue(busTemperature.getTemperature().toString());
            row.createCell(2, CellType.STRING).setCellValue(busTemperature.getHumidity());
            row.createCell(3, CellType.STRING).setCellValue(busTemperature.getWind());
        }
        try(ByteArrayOutputStream os = new ByteArrayOutputStream()){
            workbook.write(os);
            return os.toByteArray();
        } finally {
            workbook.close();
        }
    }

}
