package org.rcisoft.business.evaluate.project.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.evaluate.project.dao.ProjectEvaluateRepository;
import org.rcisoft.business.evaluate.project.service.ProjectEvaluateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

/**
 * Created by  on 2018-4-18 15:41:54.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class ProjectEvaluateServiceImpl implements ProjectEvaluateService {

    @Autowired
    private ProjectEvaluateRepository projectEvaluateRepository;


    /** 根据年份，分别查出每个项目的项目评估信息，需要页容量以及当前页数
     * @param year
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public List<Map<String, Object>> listProjectInformationByYear(String year, Integer pageNum, Integer pageSize) {
        Calendar calendar = Calendar.getInstance();
        int now = calendar.get(Calendar.YEAR);
        // 当前月份
        int month = calendar.get(Calendar.MONTH) + 1;
        if (now > Integer.parseInt(year)) {
            month = 13;
        }
        BigDecimal zero = new BigDecimal(0);
        Integer start = pageSize * (pageNum - 1);
        Integer end = pageSize * pageNum;
        List<Map<String, Object>> maps = projectEvaluateRepository.listProjectInformationByYear(year, start, end, month);
        // 循环，根据id查询月份的数量
        maps.forEach(map -> {
            // 得到节能潜力
            BigDecimal energy = new BigDecimal(map.get("energy").toString());
            // 保留两位小数
            energy = energy.setScale(2, BigDecimal.ROUND_HALF_UP);
            map.put("energy", energy);
            BigDecimal energyPotential = new BigDecimal(map.get("energyPotential") == null ? "0" : map.get("energyPotential").toString());
            /* 不需要平均值
            // 根据proId，查询月份的数量
            String proId = (String) map.get("PRO_ID");
            BigDecimal monthCount = projectEvaluateRepository.countMonth(proId, month);
            energy = energy.divide(monthCount).setScale(2, BigDecimal.ROUND_HALF_UP);
            map.put("energy", energy);
            */
            // 如果减去节能潜力的结果，大于0，赋值，如果小于0，赋值0
            energyPotential = energy.subtract(energyPotential);
            if (energyPotential.compareTo(zero) > 0)
                map.put("energyPotential", energyPotential);
            else
                map.put("energyPotential", zero);
        });
        return maps;
    }
}
