package org.rcisoft.business.mainte.adaptive.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.mainte.adaptive.service.AdaptiveService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by JiChao on 2018/4/28.
 * 运维--自适应模块
 */
@Api(tags = "自适应模块")
@RestController
@RequestMapping("adaptive")
public class AdaptiveController {

    @Autowired
    private AdaptiveService adaptiveServiceImpl;

    @ApiOperation(value="气候自适应", notes="从0点开始到23点，一共24个数")
    @ApiImplicitParams({@ApiImplicitParam(name = "proId", value = "项目主键", required = true, dataType = "字符串"),
            @ApiImplicitParam(name = "year", value = "年", required = true, dataType = "字符串或数字"),
            @ApiImplicitParam(name = "month", value = "月", required = true, dataType = "字符串或数字"),
            @ApiImplicitParam(name = "day", value = "日", required = true, dataType = "字符串或数字")
    })
    @RequestMapping("/climateAdaptation")
    public Result climateAdaptation(@RequestParam String proId, @RequestParam String year, @RequestParam String month, @RequestParam String day) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, adaptiveServiceImpl.climateAdaptation(proId, year, month, day));
    }

    @ApiOperation(value="建筑负荷自适应", notes="从0点开始到23点，一共24个数")
    @ApiImplicitParams({@ApiImplicitParam(name = "proId", value = "项目主键", required = true, dataType = "字符串"),
            @ApiImplicitParam(name = "year", value = "年", required = true, dataType = "字符串或数字"),
            @ApiImplicitParam(name = "month", value = "月", required = true, dataType = "字符串或数字"),
            @ApiImplicitParam(name = "day", value = "日", required = true, dataType = "字符串或数字")
    })
    @RequestMapping("/buildingAdaptation")
    public Result buildingAdaptation(@RequestParam String proId, @RequestParam String year, @RequestParam String month, @RequestParam String day) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, adaptiveServiceImpl.buildingAdaptation(proId, year, month, day));
    }

    /**
     * 根据ip查询气象信息
     * @return
     */
    @ApiOperation(value="根据ip查询气象信息", notes="根据ip查询气象信息")
    @RequestMapping("/weather")
    public Result weather(HttpServletRequest request, @RequestParam String city) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, adaptiveServiceImpl.weather(request, city));
    }

    @ApiOperation(value="实际温度定时任务测试", notes="实际温度定时任务测试")
    @RequestMapping("/temperature")
    public Result temperature() {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, adaptiveServiceImpl.temperature());
    }

//    @RequestMapping("/tem")
//    public Result tem(){
//        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, adaptiveServiceImpl.temperature());
//    }
}
