package org.rcisoft.business.mainte.adaptive.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.mainte.adaptive.entity.BusTemperature;
import org.rcisoft.business.mainte.adaptive.vo.Params;
import org.springframework.stereotype.Repository;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;

/**
 * Created by JiChao on 2018/4/28.
 */
@Repository
public interface BusTemperatureRepository extends Mapper<BusTemperature> {

    /**
     * 根据时间、code查询实际温度
     * @param params
     * @return
     */
    @Select("<script>select t.TM,t.TEMPERATURE from bus_temperature t " +
            "where t.CODE=#{code} and date_format(t.TM, '%Y-%c-%e')=#{time}</script>")
    @ResultMap(value = "BaseResultMap")
    List<BusTemperature> queryBusTemperatureList(Params params);

    /**
     * 批量添加每天24小时的温度
     * @param list
     * @return
     */
    @Insert("<script><foreach collection=\"list\" item=\"item\" open=\"\" close=\"\" separator=\";\">" +
            "insert into bus_temperature(TM,TEMPERATURE,CODE,HUMIDITY,WIND) values(#{item.tm},#{item.temperature},#{item.code},#{item.humidity},#{item.wind})" +
            "</foreach></script>")
    @ResultType(Integer.class)
    Integer batchSaveBusTemperature(List<BusTemperature> list);

    @Delete("<script><foreach collection=\"list\" item=\"item\" open=\"\" close=\"\" separator=\";\">" +
            "delete from bus_temperature where TM=#{item.tm} and CODE=#{item.code}" +
            "</foreach></script>")
    @ResultType(Integer.class)
    Integer batchDeleteBusTemperature(List<BusTemperature> list);

    /**
     * 查询是否有重复
     * @param code
     * @param date
     * @return
     */
    @Select("<script>select count(1) from bus_temperature where code=#{code} and date_format(TM, '%Y-%m-%d %H') = date_format(#{date}, '%Y-%m-%d %H')</script>")
    @ResultType(Integer.class)
    Integer checkExist(@Param("code") String code, @Param("date") String date);

}
