package org.rcisoft.business.mainte.distributed.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.evaluate.project.dao.ProjectEvaluateRepository;
import org.rcisoft.business.mainte.distributed.dao.MainteDistributedRepository;
import org.rcisoft.business.mainte.distributed.entity.BuildingAge;
import org.rcisoft.business.mainte.distributed.entity.DeviceAge;
import org.rcisoft.business.mainte.distributed.entity.EnergyCompared;
import org.rcisoft.business.mainte.distributed.entity.EnergyDistributed;
import org.rcisoft.business.mainte.distributed.service.MainteDistributedService;
import org.rcisoft.business.mainte.distributed.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.Calendar;
/**
 * Created by JiChao on 2018/4/23.
 */
@Service
public class MainteDistributedServiceImpl implements MainteDistributedService {

    @Autowired
    private MainteDistributedRepository mainteDistributedRepository;
    @Autowired
    private ProjectEvaluateRepository projectEvaluateRepository;

    @Override
    public List<EnergyDistributedVo> queryEnergyDistributed(String year, String month) {
        // 当月份为空，按照全年统计时，只需要统计经过的月份，未过完的月份不进行计算
        int year_month = Calendar.getInstance().get(Calendar.MONTH) + 1;
        List<EnergyDistributedVo> resultList = new ArrayList<>();
        // 如果month是空，查全年的
        /*
        *  month 如果是null  返回查找全年
        * month 不为null返回的月数小于当前月
        * */
        List<EnergyDistributed> list = mainteDistributedRepository.queryEnergyDistributed(new Params(year, month, null, year_month));
        list.forEach(energyDistributed -> {
            EnergyDistributedVo vo = new EnergyDistributedVo();
            //项目名
            vo.setName(energyDistributed.getProNm());//ProNm
            //坐标
            List<Object> value = new ArrayList<>();
            String[] coordinate = StringUtils.deleteWhitespace(energyDistributed.getBldCoordinate()).split(",");
            value.add(Float.parseFloat(coordinate[0]));
            value.add(Float.parseFloat(coordinate[1]));
            //值
            // 如果是全年的，需要查询项目的月份数量，算平均值
//            if (StringUtils.isEmpty(month)) {
//                String proId = energyDistributed.getProId();
//                BigDecimal monthCount = projectEvaluateRepository.countMonth(proId, year_month);
//                value.add(energyDistributed.getMoney().divide(monthCount).setScale(2));
//            } else {
            // 全年的求和
                value.add(energyDistributed.getMoney());
//            }
           /* value.add(new BigDecimal(0)
                    .add(energyDistributed.getWaterMoney())
                    .add(energyDistributed.getElecMoney())
                    .add(energyDistributed.getGasMoney())
                    .multiply(new BigDecimal(12))                         //乘以12       *GoGlic
                    .divide(energyDistributed.getBldArea(), 2).setScale(2)*/
                   // .divide(energyDistributed.getBldArea(), 2)

            vo.setValue(value);
            //放入结果集
            resultList.add(vo);
        });
        return resultList;
    }

    @Override
    public EnergyComparedVo queryEnergyCompared(String year, String bldZoneId) {

        //标题集
        List<Object> title = new ArrayList<>();
        //返回集
        List<EnergyComparedVo.EnergyCompareResult> resultList = new ArrayList<>();
        //结果集
        Calendar cale =Calendar.getInstance();//创建一个日历类
        int  month_now = cale.get(Calendar.MONTH)+1;//得到当前月份
        String month = String.valueOf(month_now);
        if (month.length()==1){
            month="0"+month;
        }
        System.err.println("-=-=-=-=-=-=-=-=-=-=-=-=--==-===-="+month +"-==-=-=-=-=-=-==-=-=-==-=-=-=-=-==--=-=-=-=-=-==-====--==-=");
       // List<EnergyCompared> list = mainteDistributedRepository.queryEnergyCompared(new Params(year, null, bldZoneId));
        List<EnergyCompared> list = mainteDistributedRepository.queryEnergyCompared(new Params(year,month,bldZoneId,null));
        list.forEach(e -> {

            String bldTPNm = e.getBldTPNm();
            //判断，如果title中没有这个值
            if(!title.contains(bldTPNm)){
                EnergyComparedVo.EnergyCompareResult ev = new EnergyComparedVo().new EnergyCompareResult(bldTPNm, Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}));
                //放入返回集
                resultList.add(ev);
                //放入标题集
                title.add(bldTPNm);
            }

            //取返回集中对应的记录，添加内容
            resultList.forEach(ev -> {
                if (StringUtils.equals(ev.getName(), bldTPNm))
                    ev.getData().set(e.getMon() - 1, e.getMoney());
             });
        });
        //结果
        return new EnergyComparedVo(title, resultList);
    }

    @Override
    public BuildingAgeVo queryBuildingAge() {
        // 不包括当前月份
        int month = Calendar.getInstance().get(Calendar.MONTH) + 1;
        //标题集
        List<Object> title = new ArrayList<>();
        //返回集
        List<BuildingAgeVo.BuildingAgeResult> resultList = new ArrayList<>();
        //结果集
        List<BuildingAge> list = mainteDistributedRepository.queryBuildingAge(month);
        list.forEach(b -> {
            String bldTPNm = b.getBldTPNm();
            if (!title.contains(bldTPNm)) {
                BuildingAgeVo.BuildingAgeResult buildingAgeResult = new BuildingAgeVo().new BuildingAgeResult(bldTPNm, new ArrayList<>());
                resultList.add(buildingAgeResult);
                title.add(bldTPNm);
            }
            resultList.forEach(r -> {
                if (StringUtils.equals(r.getName(), bldTPNm))
                    r.getData().add(Arrays.asList(new Object[]{b.getYear(), b.getMoney(), b.getProNm()}));

            });
        });
        //返回值
        return new BuildingAgeVo(title, resultList);
    }

    @Override
    public DeviceAgeVo queryDeviceAge() {
        // 不包括当前月份
        int month = Calendar.getInstance().get(Calendar.MONTH) + 1;
        //获得当前年份
        int year = Calendar.getInstance().get(Calendar.YEAR);
        //标题集
        List<Object> title = new ArrayList<>();
        //返回集
        List<DeviceAgeVo.DeviceAgeResult> resultList = new ArrayList<>();
        //结果集
        List<DeviceAge> list = mainteDistributedRepository.queryDeviceAge(month);
        list.forEach(d -> {
            String bldTPNm = d.getBldTPNm();
            if (!title.contains(bldTPNm)) {
                DeviceAgeVo.DeviceAgeResult deviceAgeResult = new DeviceAgeVo().new DeviceAgeResult(bldTPNm, new ArrayList<>());
                resultList.add(deviceAgeResult);
                title.add(bldTPNm);
            }
            resultList.forEach(r -> {
                if (StringUtils.equals(r.getName(), bldTPNm))
                    r.getData().add(Arrays.asList(new Object[]{year - d.getYear() + 1, d.getMoney(), d.getProNm()}));
            });
        });
        return new DeviceAgeVo(title, resultList);
    }
}
