package org.rcisoft.business.mainte.energyplan.dao;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyPlan;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;


import java.util.List;
import java.util.Map;

/**
* Created with  on 2018-4-17 17:47:37.
*/
@Repository
public interface BusEnergyPlanRepository extends BaseMapper<BusEnergyPlan> {

    /**
    *查询单个项目某一天的能好计划
    */
    @Select("<script>select a.* from bus_energy_plan a,bus_device b where a.DEV_NUM = b.DEV_NUM and" +
            " b.PRO_ID = #{proId} and date_format(a.TM, '%Y-%m-%d')=#{day} "
            + "</script>")
    List<Map<String,Object>> queryBusEnergyPlans(@Param("proId") String proId,@Param("day") String day);

    @Select("<script>SELECT sum(a.ENERGY_POWER) power_plan_day,sum(a.ENERGY_GAS) gas_plan_day FROM bus_energyplan_v a,bus_device b " +
            "WHERE a.DEV_NUM = b.DEV_NUM AND b.PRO_ID = #{pro_id} AND date_format(a.TM, '%Y-%m-%d') = #{day}"
            + "</script>")
    List<Map<String,Object>> queryEnergyPlanDay(Map<String,Object> map);

    /**
     * 查询是否是用能计划对比参数
     * @return
     */
    @Select("<script>select a.compare_flag from bus_library_param a,bus_device b " +
            "where a.DEV_NUM = b.DEV_NUM and b.DEV_NUM = #{dev_num} and a.PARAM = #{param} "
            + "</script>")
    List<Map<String,Object>> queryEnergyPlanCompareFlag(Map<String,Object> map);

    /**
     * 查询能耗计划
     * @param map
     * @return
     */
    @Select("<script>" +
            "select aa.dev_num,aa.dev_nm,GROUP_CONCAT(aa.tm) tm_str from (" +
            "select a.dev_num,b.dev_nm,a.tm from bus_energy_plan a left join bus_device b  on a.DEV_NUM = b.DEV_NUM " +
            "left join bus_device_param p on a.DEV_NUM=p.DEV_NUM and a.PARAM=p.PARAM " +
            "where DATE_FORMAT(a.tm,'%Y-%m-%d') = #{day} and b.PRO_ID = #{pro_id} order by a.tm asc) aa group by aa.dev_num" +
            "</script>")
    List<Map<String,Object>> queryEnergyPlanList(Map<String,Object> map);

    /**
     * 获取设备类型
     * @return
     */
    @Select("<script>select * from bus_device_tp where PRO_ID=#{proId}"
            + "</script>")
    List<Map<String,Object>> queryDeviceTp(@Param("proId") String proId);

    /**
     * 根据设备类型获取设备
     * @return
     */
    @Select("<script>SELECT " +
            "\ta.dev_nm,a.dev_num,a.param,a.dev_local,a.model,c.fac_nm,date_format(a.install_date,'%Y-%m-%d') install_date,b.dev_tp_nm " +
            "FROM " +
            "\tbus_device a " +
            "LEFT JOIN bus_device_tp b ON a.DEV_TP_ID = b.DEV_TP_ID " +
            "LEFT JOIN bus_factory c ON a.FAC_ID = c.FAC_ID " +
            "where a.dev_tp_id = #{dev_tp_id} and a.PRO_ID = #{proId} "
            + "</script>")
    List<Map<String,Object>> queryDeviceByTp(Map<String,Object> map);


    /**
     * 查询项目的日实际能耗
     * @param map
     * @return
     */
    @Select("<script>select sum(a.elec) power_real_day,sum(a.gas) gas_real_day from energy_count_m a,bus_device b,bus_device_meter c   " +
            "where a.MET_NUM = c.MET_NUM and c.DEV_NUM =  b.DEV_NUM and  DATE_FORMAT(a.CREATE_TIME,'%Y-%m-%d')=#{day} and b.PRO_ID = #{pro_id}"
            + "</script>")
    List<Map<String, Object>> queryBusEnergyRealCount(Map<String,Object> map);

    /**
     * 能耗计划执行结果
     * @param map
     * @return
     */
    @Select("<script>SELECT " +
            "a.dev_num,date_format(a.tm, '%Y-%m-%d %H:%i:%s') tm,a.param,a.param_val,a.source,b.original_str,c.elec " +
            "FROM " +
            "bus_energy_plan a " +
            "left join total_original b on a.TM = b.TM and a.DEV_NUM = b.DEV_NUM " +
            "left join bus_device d on a.DEV_NUM = d.DEV_NUM " +
            "left join (SELECT b.DEV_NUM,c.CREATE_TIME,sum(c.ELEC) elec FROM bus_device_meter a,bus_device b,energy_count_m c WHERE a.DEV_NUM = b.DEV_NUM AND a.MET_NUM = c.MET_NUM group by b.DEV_NUM,c.CREATE_TIME) c on a.TM = c.CREATE_TIME and a.DEV_NUM = c.DEV_NUM " +
            "WHERE " +
            "d.PRO_ID = #{pro_id} and DATE_FORMAT(a.TM, '%Y-%m-%d') = #{day}"
            + "</script>")
    List<Map<String,Object>> queryBusEnergyPlanExecute(Map<String,Object> map);

    /**
     * 根据设备编号和时间查询是否已经有能耗计划
     * @param map
     * @return
     */
    @Select("<script>select * from bus_energy_plan " +
            " where dev_num = #{dev_num} and tm = #{tm}"
            +"</script>")
    List<Map<String,Object>> queryExistEnergyPlan(Map<String,Object> map);

    /**
     * 根据设备编号和时间查询用能计划与实际功率对比
     * @param map
     * @return
     */
    @Select("<script>SELECT " +
            "\ta.dev_num,date_format(a.tm, '%Y-%m-%d %H:%i:%s') tm,a.power plan_power,c.elec*6 real_power  " +
            "from " +
            "\tbus_energy_plan a " +
            "\tleft join (SELECT b.DEV_NUM,c.CREATE_TIME,sum(c.ELEC) elec FROM bus_device_meter a,bus_device b,energy_count_m c WHERE a.DEV_NUM = b.DEV_NUM AND a.MET_NUM = c.MET_NUM group by b.DEV_NUM,c.CREATE_TIME) c on a.TM = c.CREATE_TIME and a.DEV_NUM = c.DEV_NUM " +
            "WHERE " +
            "\ta.DEV_NUM = #{dev_num} and DATE_FORMAT(a.TM, '%Y-%m-%d') = #{day} " +
            "order by tm asc"
            +"</script>")
    List<Map<String,Object>> queryEnergyPowerCompare(Map<String,Object> map);

    /**
     * 更改设备参数库录入开关状态
     * @param map
     */
    @Select("<script>update bus_device set pl_switch = #{pl_switch} where dev_num = #{dev_num}" +
            "</script>")
    void updateDevicePswitch(Map<String,Object> map);

    /**
     * 参数库数据删除
     * 理论上对于一个设备来说，一年只做一次的参数库录入，为防止数据重复，在打开录入开关前，
     * 删除今年的录入数据。
     */
    @Select("<script>delete from bus_param_library where dev_num = #{dev_num} and year = #{year}" +
            "</script>")
    void deleteParamLibrary(Map<String,Object> map);

    @Select("<script>select date_format(tm, '%Y-%m-%d %H:%i:%s') tm,sensor_json from total_sensor  " +
            "where PRO_ID = #{pro_id} and DATE_FORMAT(tm,'%Y-%m-%d') = #{day}" +
            "</script>")
    List<Map<String,Object>> querySensor(Map<String,Object> map);

    /**
     * 根据设备编号和日期删除某一天该设备的用能计划
     * @param map
     * @return
     */
    @Select("<script>delete from bus_energy_plan where dev_num = #{dev_num} and DATE_FORMAT(tm,'%Y-%m-%d') = #{day}</script>")
    void deleteEnergyPlanDay(Map<String,Object> map);

    /**
     * 删除项目某一天的能耗计划
     * @param day
     * @param proId
     */
    @Select("<script>delete from bus_energy_plan where DATE_FORMAT(tm,'%Y-%m-%d') = #{day} and dev_num in (select dev_num from bus_device where pro_id = #{proId})</script>")
    void deleteEnergyPlan(@Param("day") String day,@Param("proId") String proId);
}

