package org.rcisoft.business.mainte.energyplan.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.rcisoft.business.mainte.energyplan.dao.BusEnergyPlanRepository;
import org.rcisoft.business.mainte.energyplan.dao.BusEnergyplanTRepository;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyPlan;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyplanT;
import org.rcisoft.business.mainte.energyplan.service.BusEnergyPlanService;
import org.rcisoft.business.mainte.energyplan.service.BusEnergyplanTService;
import org.rcisoft.business.mainte.energyplan.vo.EnergyPlanList;
import org.rcisoft.business.mainte.energyplan.vo.EnergyPlanVo;
import org.rcisoft.business.mainte.paramlibrary.dao.ParamLibraryRepository;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;



import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import tk.mybatis.mapper.entity.Example;

/**
 * Created by  on 2018-4-17 17:47:37.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusEnergyPlanServiceImpl implements BusEnergyPlanService {

    @Autowired
    private BusEnergyPlanRepository busEnergyPlanRepository;
    @Autowired
    private ParamLibraryRepository paramLibraryRepository;
    @Autowired
    private BusEnergyplanTService busEnergyplanTService;
    @Autowired
    private BusEnergyplanTRepository busEnergyplanTRepository;
    @Value("${serverimgurl}")
    private String serverImgUrl;


    /**
    * 保存 busEnergyPlan
    * @param
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(EnergyPlanVo energyPlanVo)throws Exception{
        BusEnergyPlan busEnergyPlan = new BusEnergyPlan();
        //时间拆分，10分钟一条记录
        ArrayList timeArr = new ArrayList();
        Calendar c = Calendar.getInstance();
        SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date sTime = simple.parse(energyPlanVo.getStartTm());
        Date eTime = simple.parse(energyPlanVo.getEndTm());
        // 如果开始时间 > 结束时间，直接返回
        if (sTime.compareTo(eTime) > 0) return new PersistModel(0);
        while(!sTime.equals(eTime)){
            timeArr.add(sTime);
            c.setTime(sTime);
            c.add(Calendar.MINUTE,10);
            sTime = c.getTime();
        }
        // 赋值参数code，参数值，来源
        busEnergyPlan.setParam(energyPlanVo.getParamStr());
        busEnergyPlan.setParamVal(energyPlanVo.getParamValStr());
        busEnergyPlan.setSource(energyPlanVo.getSource());

        for(Object tm : timeArr){
            busEnergyPlan.setDevNum(energyPlanVo.getDevNum());
            busEnergyPlan.setTm((Date)tm);
            // 赋值功率、用气速率
            busEnergyPlan.setPower(energyPlanVo.getPower());
            busEnergyPlan.setGasSpeed(energyPlanVo.getGasSpeed());

            // 保存
            busEnergyPlanRepository.insertSelective(busEnergyPlan);

        }
        return new PersistModel(1);
    }

    @Transactional
    @Override
    public PersistModel saveOrUpdate(EnergyPlanList energyPlanlist) throws Exception {
        List<BusEnergyplanT> busEnergyplanTList = energyPlanlist.getBusEnergyplanTList();
        int line = 1;
        String message = "";
        // ----判断时间区间，如果出现时间段重复的情况，不进行操作----
        boolean dateFlag = true;
        List<Map<String, Date>> dateList = new ArrayList<>();
        List<EnergyPlanVo> epList = energyPlanlist.getEnergyPlanVoList();
        // 循环前台传来的对象
        for1 : for (EnergyPlanVo energyPlanVo : epList) {
            try {
                Date startTm = DateUtils.parseDate(energyPlanVo.getStartTm(), "yyyy-MM-dd HH:mm:ss");
                Date endTm = DateUtils.parseDate(energyPlanVo.getEndTm(), "yyyy-MM-dd HH:mm:ss");
                if (dateList.size() > 0) {
                    // 循环list，与最新的记录做对比
                    for (Map<String, Date> stringDateMap : dateList) {
                        Date startTm1 = stringDateMap.get("startTm");
                        Date endTm1 = stringDateMap.get("endTm");
                        // 符合要求的情况下，继续下一条
                        if (endTm.compareTo(startTm1) <= 0 || startTm.compareTo(endTm1) >= 0) {
                            continue ;
                        } else { // 如果时间不符合要求，跳出循环
                            dateFlag = false;
                            break for1;
                        }
                    }
                }
                // dateList为0，或者没有跳出循环的情况下，将新的记录添加到dateList中
                Map<String, Date> map = new HashMap<>();
                map.put("startTm", startTm);
                map.put("endTm", endTm);
                dateList.add(map);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        // 如果dateFlag = false，直接返回错误提示
        if (!dateFlag)
            return new PersistModel(0, "多条记录的时间范围不能重叠");
        // ------------------------------------------------------
        String devNum = energyPlanlist.getDevNum();
        String day = energyPlanlist.getDay();
        //先清除掉该设备该天的能耗计划后重新新增
        PersistModel pm = remove(devNum,day);
        Boolean flag = true;
        if(pm.isSuccessBySinglePersist()){
            flag = busEnergyplanTService.remove(energyPlanlist.getDevNum(),energyPlanlist.getDay());
        }
        if(flag){
            PersistModel p = null;
                try{
                    for(EnergyPlanVo energyPlanVo : epList) {
                        p = save(energyPlanVo);
                    }
                    if(p.isSuccessBySinglePersist()){
                        busEnergyplanTService.save(busEnergyplanTList);
                    }
                }catch(Exception e){
                    line = 0;
                    message = "系统错误，操作失败";
                }
        }else{
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            message = "系统错误，操作失败";
        }
        return new PersistModel(line,message);
    }

    /**
    * 逻辑删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(String devNum,String day){
        int line = 1;
        Map<String,Object> map = new HashMap<>();
        map.put("dev_num",devNum);
        map.put("day",day);
        try {
            busEnergyPlanRepository.deleteEnergyPlanDay(map);
        }catch(Exception e){
            line = 0;
        }
        return new PersistModel(line);
    }



    @Override
    public List<Map<String, Object>> queryEnergyPlanDay(String proid, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        return busEnergyPlanRepository.queryEnergyPlanDay(map);
    }

    @Override
    public List<Map<String, Object>> queryEnergyPlanList(String proid, String day)throws Exception {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        List<Map<String, Object>> list = busEnergyPlanRepository.queryEnergyPlanList(map);
        if (list != null && list.size() > 0) {
            // 如果只有一个值，并且都为null，直接返回一个空数组
            Map<String, Object> map1 = list.get(0);
            if (map1.get("dev_nm") == null && map1.get("tm_str") == null && map1.get("dev_num") == null) return new ArrayList<>();

            ArrayList timeArr = new ArrayList();
            Calendar c = Calendar.getInstance();
            SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date sTime = simple.parse(day + " 00:00:00");
            Date eTime = simple.parse(day + " 23:50:00");
            while(!sTime.equals(eTime)){
                timeArr.add(simple.format(sTime));
                c.setTime(sTime);
                c.add(Calendar.MINUTE,10);
                sTime = c.getTime();
            }
                timeArr.add(day + " 23:50:00");
            for(Map<String, Object> tmap : list){
                Map<String,Object> tmMap = new HashMap<String,Object>();
                for(Object tm : timeArr){
                    tmMap.put(tm.toString(),null);
                }
                String[] ts = tmap.get("tm_str").toString().split(",");
                for(String s : ts){
                    tmMap.put(s,"1");
                }
                tmap.put("planmap",tmMap);
                tmap.put("planT",busEnergyplanTService.queryBusEnergyplanTs(tmap.get("dev_num").toString(),day));
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> queryDeviceTp(String proId) {
        List<Map<String, Object>> list = busEnergyPlanRepository.queryDeviceTp(proId);
//        for(Map<String, Object> m : list){
            /*if(m.get("DEV_TP_IMG")!=null && !m.get("DEV_TP_IMG").toString().equals("") && !m.get("DEV_TP_IMG").toString().equals("null")){
                String type = "";
                String img = UploadUtil.GetImageStr(m.get("DEV_TP_IMG").toString());
                m.put("IMG", img);
            }else{
                m.put("IMG","");
            }*/
//        }
        return list;
    }

    @Override
    public List<Map<String, Object>> queryDeviceByTp(String dev_tp_id, String proId) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_tp_id",dev_tp_id);
        map.put("proId",proId);
        return busEnergyPlanRepository.queryDeviceByTp(map);
    }

    @Override
    public List<Map<String, Object>> queryBusEnergyRealCount(String proid, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        return busEnergyPlanRepository.queryBusEnergyRealCount(map);
    }

    @Override
    public List<Map<String, Object>> queryEnergyPlanExecute(String proid, String day)throws Exception {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        Map<String,Object> sensorMap = new HashMap<>();
        List<Map<String,Object>> sensorList = busEnergyPlanRepository.querySensor(map);
        for(Map<String,Object> m : sensorList){
            sensorMap.put(m.get("tm").toString(),m.get("sensor_json"));
        }
        List<Map<String,Object>> executeList = busEnergyPlanRepository.queryBusEnergyPlanExecute(map);
        Map<String,Object> executeMap = new HashMap<String,Object>();
        for(Map<String,Object> tmap : executeList){
            //设备状态 0：状态不一致 1：参数不一致  2：完全一致
            //状态由耗电量决定
            String flag = "2";
            //10分钟内设备耗电量为空或小于0.5度，设备默认为关闭状态
            if(tmap.get("elec")==null || tmap.get("elec").toString().equalsIgnoreCase("") || Float.valueOf(tmap.get("elec").toString()) <= 0.5){
                flag = "0";
            }else{
                // 循环参数code，判断状态
                String param_all = tmap.get("param").toString();
                String param_val_all = tmap.get("param_val").toString();
                String param_source_all = tmap.get("source").toString();
                String[] param_split = param_all.split("/");
                String[] param_val_split = param_val_all.split("/");
                String[] param_source_split = param_source_all.split("/");
                for (int i = 0; i < param_split.length; i++) {
                    String param = param_split[i];
                    String param_val = param_val_split[i];
                    // 参数状态
                    String param_source = param_source_split[i];
                    String compare_val = "";
                    if(param_source.equals("2")){//要对比的参数取自于传感器
                        if(sensorMap.get(tmap.get("tm").toString())==null){
                            compare_val = "";
                        }else{
                            String sensorJson = sensorMap.get(tmap.get("tm")).toString();
                            if (StringUtils.isNotEmpty(sensorJson)) {
                                JSONObject sensorBody = JSONArray.parseObject(sensorJson);
                                // 保留一位小数做判断
                                BigDecimal p_v = new BigDecimal(sensorBody.get(param)==null?"0":sensorBody.get(param).toString());
                                compare_val = p_v.setScale(1).toString();
                            }
                        }

                    }else{//要对比的参数取自于设备本身
                        String original = (String) tmap.get("original_str");
                        if (StringUtils.isNotEmpty(original)) {
                            JSONObject original_JSON = JSONArray.parseObject(original);
                            BigDecimal p_v = new BigDecimal(original_JSON.get(param)==null?"0":original_JSON.get(param).toString());
                            compare_val = p_v.setScale(1).toString();
                        }
                    }
                    // 如果没有取到对比的参数
                    if (StringUtils.isEmpty(compare_val)) {
                        flag = "1";
                    } else {
                        // 将两个对比的参数转换成数字进行对比
                        if (new BigDecimal(param_val).compareTo(new BigDecimal(compare_val)) != 0) {
                            flag = "1";
                        }
                    }
                }
            }
            executeMap.put(tmap.get("dev_num").toString()+","+tmap.get("tm").toString(),flag);
            SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String now = simple.format(new Date());
            if(simple.parse(tmap.get("tm").toString()).getTime()>simple.parse(now).getTime()){
                executeMap.put(tmap.get("dev_num").toString()+","+tmap.get("tm").toString(),"-1");
            }
        }
        //根据code，获得参数的名称和单位

        List<Map<String, Object>> planList = queryEnergyPlanList(proid,day);
        for(Map<String, Object> plan : planList){
            String dev_num = plan.get("dev_num").toString();
            Map<String, Object> planMap = (Map<String, Object>)plan.get("planmap");
            for (Map.Entry<String, Object> entry : planMap.entrySet()) {
                planMap.put(entry.getKey(),executeMap.get(dev_num+","+entry.getKey()));
            }
        }
        return planList;
    }

    @Override
    public List<Map<String, Object>> queryEnergyPowerCompare(String dev_num, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("day",day);
        return busEnergyPlanRepository.queryEnergyPowerCompare(map);
    }


    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int paramLibraryOperate(Map<String, Object> map) {
        String pl_switch = map.get("pl_switch").toString();
        if(pl_switch.equalsIgnoreCase("1")){//参数库录入：开
            SimpleDateFormat simple = new SimpleDateFormat("yyyy");
            map.put("year",simple.format(new Date()));
            busEnergyPlanRepository.deleteParamLibrary(map);
        }
        busEnergyPlanRepository.updateDevicePswitch(map);
        return 1;
    }

    @Transactional
    @Override
    public PersistModel energyPlanCopy(String proId, String day, String copyDays) {
        int line = 1;
        String message = "";
        if(proId==null || proId.equals("") || day==null || day.equals("")){
            line = 0;
            message = "能耗计划复制失败";
            return new PersistModel(line,message);
        }
        String[] copyDayList = copyDays.split(",");
        //能耗计划
        List<Map<String,Object>> planList = busEnergyPlanRepository.queryBusEnergyPlans(proId, day);
        //能耗计划概览
        List<Map<String,Object>> plantList = busEnergyplanTRepository.queryBusEnergyPlanTs(proId, day);
        SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try{
            for(String newDay : copyDayList){
                if(deleteEnergyDay(newDay, proId)){
                    for(Map<String,Object> mp : planList){
                        BusEnergyPlan be = new BusEnergyPlan();
                        be.setDevNum(mp.get("DEV_NUM").toString());
                        be.setParam(mp.get("PARAM").toString());
                        be.setParamVal(mp.get("PARAM_VAL").toString());
                        be.setPower(mp.get("POWER") == null ? null : new BigDecimal(mp.get("POWER").toString()));
                        be.setGasSpeed(mp.get("GAS_SPEED") == null ? null : new BigDecimal(mp.get("GAS_SPEED").toString()));
                        be.setTm(simple.parse(newDay +" "+ simple.format(mp.get("TM")).substring(11)));
                        busEnergyPlanRepository.insertSelective(be);
                    }
                    for(Map<String,Object> mp : plantList){
                        BusEnergyplanT bt = new BusEnergyplanT();
                        bt.setDevNum(mp.get("DEV_NUM").toString());
                        bt.setDevTpId(mp.get("DEV_TP_ID").toString());
                        bt.setStartTm(newDay +" "+ mp.get("START_TM").toString().substring(11));
                        bt.setEndTm(newDay +" "+ mp.get("END_TM").toString().substring(11));
                        bt.setValOne(mp.get("VAL_ONE").toString());
                        bt.setValTwo(mp.get("VAL_TWO") == null ? null : mp.get("VAL_TWO").toString());
                        bt.setValThree(mp.get("VAL_THREE") == null ? null : mp.get("VAL_THREE").toString());
                        bt.setPower(mp.get("POWER") == null ? null : mp.get("POWER").toString());
                        bt.setGasSpeed(mp.get("GAS_SPEED") == null ? null : mp.get("GAS_SPEED").toString());
                        bt.setCompareVal(mp.get("COMPARE_VAL").toString());
                        bt.setId(UUID.randomUUID().toString().replace("-",""));
                        bt.setDay(newDay);
                        busEnergyplanTRepository.insert(bt);
                    }
                }else{
                    message = "能耗计划清除失败，无法复制";
                }
            }
        }catch(Exception e){
            e.printStackTrace();
            line = 0;
            message = "数据异常，能耗计划复制失败";
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return new PersistModel(line,message);
    }

    @Override
    public Boolean deleteEnergyDay(String day, String proId) {
        try{
            busEnergyPlanRepository.deleteEnergyPlan(day, proId);
            busEnergyplanTRepository.deleteEnergyPlanT(day, proId);
        }catch(Exception e){
            e.printStackTrace();
            return false;
        }
        return true;
    }


}
