package org.rcisoft.business.mainte.fault.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.mainte.fault.entity.BusDeviceFault;
import org.rcisoft.business.mainte.fault.vo.BusDeviceFaultVo;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
* Created with  on 2018-4-17 11:19:46.
*/
@Repository
public interface BusDeviceFaultRepository extends BaseMapper<BusDeviceFault> {

    /**
     * 故障次数
     * @param busDeviceFaultVo
     * @return 日期，次数的list
     */
    @Select("<script>select date_format(b.TM,'%e') `day`,count(*) NUM from bus_device_fault b where b.PRO_ID=#{proId} and date_format(b.TM,'%Y-%c')=#{date} group by date_format(b.TM,'%e')+0 asc</script>")
    @ResultMap(value = "vo")
    List<BusDeviceFaultVo> statFault(BusDeviceFaultVo busDeviceFaultVo);

    /**
     * 故障统计列表
     * @param busDeviceFaultVo
     * @return
     */
    @Select("<script>select date_format(f.TM,'%Y-%m-%d') `DATE`,d.DEV_NM,dp.PARAM_NM,f.CONTENT from bus_device_fault f,bus_device d,bus_device_param p,bus_devicetp_param dp \n" +
            "where f.DEV_NUM=d.DEV_NUM and f.DEV_NUM=p.DEV_NUM and f.P_CODE=p.PARAM and p.PARAM_CODE=dp.PARAM_CODE \n" +
            "and dp.PRO_ID=#{proId} and date_format(f.TM,'%Y-%c')=#{date}</script>")
    @ResultMap(value = "faultList")
    List<BusDeviceFaultVo> queryFaultList(BusDeviceFaultVo busDeviceFaultVo);

    /**
     * 查找是否已经有错误参数的记录
     * @param devNum
     * @param pcode
     * @param tm
     * @return
     */
    @Select("<script>select count(*) from bus_device_fault where DEV_NUM=#{devNum} and P_CODE=#{pcode} and date_format(TM,'%Y-%m-%d') = date_format(#{tm},'%Y-%m-%d')</script>")
    @ResultType(java.lang.Integer.class)
    Integer faultExist(@Param("devNum") String devNum, @Param("pcode") String pcode, @Param("tm") String tm);

    /**
     * 批量保存
     * @param list
     * @return
     */
    @Insert("<script><foreach collection=\"list\" item=\"item\" separator=\";\">" +
            "insert into bus_device_fault(DEV_NUM,TM,P_CODE,CONTENT,PRO_ID) " +
            "values(#{item.devNum},#{item.tm},#{item.pCode},#{item.content},#{item.proId})" +
            "</foreach></script>")
    @ResultType(java.lang.Integer.class)
    Integer saveFaultList(List<BusDeviceFault> list);
}

