package org.rcisoft.business.manage.controller;


/*固定导入*/
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.service.BusProjectService;
import org.rcisoft.business.manage.vo.ExcelParams;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.JwtUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.WebRequest;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
* Created by  on 2018-5-2 16:09:30.
*/
@RestController
@RequestMapping("manage/busproject")
public class BusProjectController extends PaginationController<BusProject> {

    @Autowired
    private BusProjectService busProjectServiceImpl;


    @ApiOperation(value="添加项目", notes="添加项目")
    @RequestMapping(value = "/add")
    public Result add(@Valid BusProject busProject) {
        PersistModel data = busProjectServiceImpl.save(busProject);
        return Result.builder(data);
    }


    @ApiOperation(value="修改项目信息", notes="修改项目信息")
    @PostMapping("/update")
    public Result update(@Valid BusProject busProject) {
        PersistModel data = busProjectServiceImpl.merge(busProject);
        return Result.builder(data);
    }

    @ApiOperation(value="删除项目", notes="删除项目")
    @RequestMapping("/delete")
    public Result delete(@RequestParam String proId) {
        BusProject busProject = new BusProject();
        busProject.setProId(proId)  ;
        PersistModel data = busProjectServiceImpl.delete(busProject);
        return Result.builder(data);
    }

    @ApiOperation(value="根据条件查询单个项目信息", notes="根据条件查询单个项目信息")
    @RequestMapping("/detail")
    public Result detail(@Valid BusProject busProject) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busProjectServiceImpl.findByParam(busProject));
    }

    @ApiOperation(value="查看  集合", notes="查看  集合")
    @RequestMapping(value = "/queryBusProjectByPagination")
    public GridModel listByPagination(BusProject busProject) {
        busProjectServiceImpl.findAllByPagination(getPaginationUtility(), busProject);
        return getGridModelResponse();
    }

    @ApiOperation(value="查询用户权限项目", notes="查询用户权限项目")
    @RequestMapping(value = "/queryBusProjectByUser")
    public Result queryBusProjectByUser() {
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        Map<String,Object> map = new HashMap<>();
        map.put("user_id",userId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busProjectServiceImpl.queryBusProjectByUser(map));
    }

    @ApiOperation(value="更改 是否接收数据 标志位", notes="更改 是否接收数据 标志位")
    @RequestMapping(value = "/receiveData")
    public Result receiveData(@RequestParam String proId, @RequestParam Integer receiveData) {
        Integer result = busProjectServiceImpl.receiveData(proId, receiveData);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="根据输入的参数导出excel", notes="根据输入的参数导出excel")
    @RequestMapping(value = "/downloadExcel")
    public void downloadExcel(HttpServletResponse response, @RequestParam String start, @RequestParam String end, @RequestParam Integer type, @RequestParam String code, @RequestParam String proId) {
        busProjectServiceImpl.downloadExcel(response, new ExcelParams(proId, start, end, code, type));
    }

    @ApiOperation(value="检查网关数据", notes="返回值，1：正确，0：有错误")
    @RequestMapping(value = "/checkData")
    public Result checkData(@RequestParam String proId) {
        Integer result = busProjectServiceImpl.checkData(proId);
        return Result.builder(new PersistModel(result), null, "网关数据接收异常", null);
    }

    @InitBinder
    public void initBinder(WebDataBinder webDataBinder, WebRequest webRequest) {
        DateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        webDataBinder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
    }
}
