package org.rcisoft.business.manage.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.manage.entity.BusSaving;
import org.rcisoft.business.manage.service.BusSavingService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * Created by JiChao on 2018/5/17.
 */
@Api(tags = "评估--节能改造--认定")
@RestController
@RequestMapping("bussaving")
public class BusSavingController extends PaginationController<BusSaving> {

    @Autowired
    private BusSavingService busSavingServiceImpl;

    @ApiOperation(value="增加", notes="增加")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "examiner", value = "检定员", dataType = "字符串"),
            @ApiImplicitParam(name = "qualification", value = "执业资质", dataType = "字符串"),
            @ApiImplicitParam(name = "tm", value = "从业时间", dataType = "字符串"),
            @ApiImplicitParam(name = "performance", value = "项目业绩", dataType = "字符串")
    })
    @RequestMapping("/save")
    public Result save(BusSaving busSaving) {
        Integer result = busSavingServiceImpl.save(busSaving);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", required = true, dataType = "字符串"),
            @ApiImplicitParam(name = "examiner", value = "检定员", dataType = "字符串"),
            @ApiImplicitParam(name = "qualification", value = "执业资质", dataType = "字符串"),
            @ApiImplicitParam(name = "tm", value = "从业时间", dataType = "字符串"),
            @ApiImplicitParam(name = "performance", value = "项目业绩", dataType = "字符串")
    })
    @PostMapping("/update")
    public Result update(@Valid BusSaving busSaving) {
        Integer result = busSavingServiceImpl.update(busSaving);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="删除", notes="删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, dataType = "字符串")
    })
    @RequestMapping("/delete/{id:\\w+}")
    public Result delete(@PathVariable String id) {
        Integer result = busSavingServiceImpl.delete(id);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="分页查询", notes="分页查询")
    @RequestMapping("/findAllByPagination")
    public GridModel findAllByPagination() {
        busSavingServiceImpl.findAllByPagination(getPaginationUtility());
        return getGridModelResponse();
    }

    @ApiOperation(value="根据id查找", notes="根据id查找")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, dataType = "字符串")
    })
    @RequestMapping("/findBusSaving/{id:\\w+}")
    public Result findBusSaving(@PathVariable String id) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, busSavingServiceImpl.findBusSaving(id));
    }

}
