package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.manage.dao.BusDeviceTpRepository;
import org.rcisoft.business.manage.entity.BusDeviceTp;
import org.rcisoft.business.manage.entity.BusDevicetpParam;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.service.BusDeviceTpService;
import org.rcisoft.business.manage.service.BusDevicetpParamService;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-2 16:09:29.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusDeviceTpServiceImpl implements BusDeviceTpService {

    @Autowired
    private BusDeviceTpRepository busDeviceTpRepository;
    @Autowired
    private BusDevicetpParamService busDevicetpParamService;
    @Value("${filepath.devicetp}")
    private String filepath;


    /**
    * 保存 busDeviceTp
    * @param busDeviceTp
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusDeviceTp busDeviceTp){
        int line = 0;
        // 验重
        Integer repeat = this.repeat(null, busDeviceTp.getDevTpNm(), busDeviceTp.getProId());
        if (repeat > 0)
            return new PersistModel(line, "设备类型名称不能重复");
        //增加操作
        busDeviceTp.setDevTpId(UUID.randomUUID().toString().replace("-",""));
        line = busDeviceTpRepository.insertSelective(busDeviceTp);
        return new PersistModel(line);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(BusDeviceTp busDeviceTp) {
        int line = 0;
        String message = "";
        Example example = new Example(BusDeviceTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devTpId", busDeviceTp.getDevTpId());
        Map<String,Object> map = new HashMap<>();
        map.put("dev_tp_id",busDeviceTp.getDevTpId());
        if(busDeviceTp.getDevTpId()!=null && !busDeviceTp.getDevTpId().equals("")){
            if(busDeviceTpRepository.queryBuildTpParam(map)==null || busDeviceTpRepository.queryBuildTpParam(map).size()<1){
                line = busDeviceTpRepository.deleteByExample(example);
            }else{
                message = "关联模板不为空，无法删除";
            }
        }else{
            message = "ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }


    /**
    * 修改 busDeviceTp
    * @param busDeviceTp
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusDeviceTp busDeviceTp){
        int line = 0;
        if (StringUtils.isNotEmpty(busDeviceTp.getDevTpNm())) {
            // 验重
            Integer repeat = this.repeat(busDeviceTp.getDevTpId(), busDeviceTp.getDevTpNm(), busDeviceTp.getProId());
            if (repeat > 0)
                return new PersistModel(line, "设备类型名称不能重复");
        }

        Example example = new Example(BusDeviceTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devTpId", busDeviceTp.getDevTpId());
        line = busDeviceTpRepository.updateByExampleSelective(busDeviceTp,example);
        return new PersistModel(line);
    }

    @Override
    public List<Map<String,Object>> queryBusDeviceTpList(String proId) {
        List<Map<String,Object>> list =  busDeviceTpRepository.queryBusDeviceTps(proId);
        for(Map<String,Object> tmap : list){
            BusDevicetpParam bdp = new BusDevicetpParam();
            bdp.setDevTpId(tmap.get("DEV_TP_ID").toString());
            bdp.setProId(proId);
            tmap.put("paramList",busDevicetpParamService.queryBusDevicetpParam(bdp));
        }
        return list;
    }

    @Override
    public PersistModel upload(MultipartFile file, String devTpId) {
        int line = 1;
        String fileName = file.getOriginalFilename();
        String suffixName = fileName.substring(fileName.lastIndexOf("."));
        // 查找之前的image文件
        BusDeviceTp busDeviceTp1 = new BusDeviceTp();
        busDeviceTp1.setDevTpId(devTpId);
        BusDeviceTp busDeviceTp2 = busDeviceTpRepository.selectOne(busDeviceTp1);
        String devTpImg = busDeviceTp2.getDevTpImg();
        //文件路径
        String uuid = UUID.randomUUID().toString();
        String savePath = filepath + uuid + suffixName;
        File saveFile = new File(filepath + devTpImg);
        if(saveFile.exists()){
            saveFile.delete();
        }
        try{
            FileUtils.copyInputStreamToFile(file.getInputStream(), new File(savePath));
            BusDeviceTp busDeviceTp = new BusDeviceTp();
            busDeviceTp.setDevTpId(devTpId);
            busDeviceTp.setDevTpImg(uuid + suffixName);
            merge(busDeviceTp);
            line = 1;
        }catch(Exception e){
            e.printStackTrace();
        }
        return new PersistModel(line);
    }

    /**
     * 验重
     * @param devTpNm
     * @return
     */
    private Integer repeat(String devTpId, String devTpNm, String proId) {
        Example example = new Example(BusDeviceTp.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devTpNm", devTpNm);
        criteria.andEqualTo("proId", proId);
        if (StringUtils.isNotEmpty(devTpId))
            criteria.andNotEqualTo("devTpId", devTpId);
        return busDeviceTpRepository.selectCountByExample(example);
    }

}
