package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.manage.dao.BusFactoryRepository;
import org.rcisoft.business.manage.entity.BusFactory;
import org.rcisoft.business.manage.service.BusFactoryService;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-16 14:34:55.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusFactoryServiceImpl implements BusFactoryService {

    @Autowired
    private BusFactoryRepository busFactoryRepository;


    /**
    * 保存 busFactory
    * @param busFactory
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusFactory busFactory){
        int line = 0;
        // 验空
        String facNm = busFactory.getFacNm();
        if (StringUtils.isEmpty(facNm))
            return new PersistModel(line, "厂家名称不能为空");
        // 验重
        Integer repeat = this.repeat(null, facNm);
        if (repeat > 0)
            return new PersistModel(line, "厂家名称不能重复");

        //增加操作
        busFactory.setFacId(UUID.randomUUID().toString().replace("-",""));
        line = busFactoryRepository.insertSelective(busFactory);
        return new PersistModel(line);
    }
    /**
    * 修改 busFactory
    * @param busFactory
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusFactory busFactory){
        Integer line = 0;
        String message = "";
        // 验空
        String facNm = busFactory.getFacNm();
        if (StringUtils.isEmpty(facNm))
            return new PersistModel(line, "厂家名称不能为空");
        // 验重
        Integer repeat = this.repeat(busFactory.getFacId(), busFactory.getFacNm());
        if (repeat > 0)
            return new PersistModel(line, "厂家名称重复");

        Example example = new Example(BusFactory.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("facId",busFactory.getFacId());
        if(busFactory.getFacId()!=null && !busFactory.getFacId().equals("")){
            line = busFactoryRepository.updateByExampleSelective(busFactory,example);
        }else{
            message = "ID为空，更新失败";
        }
        return new PersistModel(line,message);
    }

    @Override
    public PersistModel delete(String id) {
        Integer line = 0;
        String message = "";
        Example example = new Example(BusFactory.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("facId",id);
        if(id!=null && !id.equals("")){
            line = busFactoryRepository.deleteByExample(example);
        }else{
            message = "ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    @Override
    public List<Map<String,Object>> queryBusFactoryList() {
        return busFactoryRepository.queryBusFactorys();
    }

    /**
     * 验证厂家名称是否重复
     * @param factoryName
     * @return
     */
    private Integer repeat(String facId, String factoryName) {
        Example example = new Example(BusFactory.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("facNm", factoryName);
        if (StringUtils.isNotEmpty(facId))
            criteria.andNotEqualTo("facId", facId);
        return busFactoryRepository.selectCountByExample(example);
    }


}
