package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.mainte.energyplan.dao.BusEnergyPlanRepository;
import org.rcisoft.business.mainte.energyplan.dao.BusEnergyplanTRepository;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyPlan;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyplanT;
import org.rcisoft.business.mainte.paramlibrary.dao.ParamLibraryRepository;
import org.rcisoft.business.mainte.paramlibrary.entity.BusParamLibrary;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.business.manage.entity.BusLibraryParam;
import org.rcisoft.business.manage.service.BusLibraryParamService;
import org.rcisoft.business.manage.vo.BusLibraryParamList;
import org.rcisoft.business.overview.dao.BusEnergyplanVRepository;
import org.rcisoft.business.overview.entity.BusEnergyplanV;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-2 16:09:30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusLibraryParamServiceImpl implements BusLibraryParamService {

    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;
    @Autowired
    private ParamLibraryRepository busParamLibraryRepository;
    @Autowired
    private BusEnergyPlanRepository busEnergyPlanRepository;
    @Autowired
    private BusEnergyplanTRepository busEnergyplanTRepository;
    @Autowired
    private BusEnergyplanVRepository busEnergyplanVRepository;

    /**
    * 保存 busLibraryParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusLibraryParamList busLibraryParamList){
        int line = 0;
        List<BusLibraryParam> busLibraryParams = busLibraryParamList.getBusLibraryParamList();
        for(BusLibraryParam bp : busLibraryParams){
            bp.setId(UUID.randomUUID().toString().replace("-",""));
            line  = busLibraryParamRepository.insertSelective(bp);
        }
        return new PersistModel(line);
    }

    /**
    * 删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devNum){
        // 1.根据dev_num删除bus_library_param
        Example example_BusLibraryParam = new Example(BusLibraryParam.class);
        Example.Criteria criteria_BusLibraryParam = example_BusLibraryParam.createCriteria();
        criteria_BusLibraryParam.andEqualTo("devNum", devNum);
        busLibraryParamRepository.deleteByExample(example_BusLibraryParam);
        // 2.根据dev_num删除bus_param_library
        Example example_BusParamLibrary = new Example(BusParamLibrary.class);
        Example.Criteria criteria_BusParamLibrary = example_BusParamLibrary.createCriteria();
        criteria_BusParamLibrary.andEqualTo("devNum", devNum);
        busParamLibraryRepository.deleteByExample(example_BusParamLibrary);
        // 3.根据dev_num删除bus_energy_plan
        Example example_BusEnergyPlan = new Example(BusEnergyPlan.class);
        Example.Criteria criteria_BusEnergyPlan = example_BusEnergyPlan.createCriteria();
        criteria_BusEnergyPlan.andEqualTo("devNum", devNum);
        busEnergyPlanRepository.deleteByExample(example_BusEnergyPlan);
        // 4.根据dev_num删除bus_energyplan_t
        Example example_BusEnergyplanT = new Example(BusEnergyplanT.class);
        Example.Criteria criteria_BusEnergyplanT = example_BusEnergyplanT.createCriteria();
        criteria_BusEnergyplanT.andEqualTo("devNum", devNum);
        busEnergyplanTRepository.deleteByExample(example_BusEnergyplanT);
        // 5.根据dev_num删除bus_energyplan_v
        Example example_BusEnergyplanV = new Example(BusEnergyplanV.class);
        Example.Criteria criteria_BusEnergyplanV = example_BusEnergyplanV.createCriteria();
        criteria_BusEnergyplanV.andEqualTo("devNum", devNum);
        busEnergyplanVRepository.deleteByExample(example_BusEnergyplanV);

        return new PersistModel(1, "操作成功");
    }

    /**
    * 修改 busLibraryParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusLibraryParamList busLibraryParamList){
        String message = "";
        PersistModel pm = delete(busLibraryParamList.getBusLibraryParamList().get(0).getDevNum());
        if(pm.isSuccessBySinglePersist()){
            return save(busLibraryParamList);
        }else{
            message = "设备参数库不为空，无法更新";
            return new PersistModel(0,message);
        }
    }

    @Override
    public List<Map<String, Object>> queryLibraryParamByDev(String devNum) {
        return busLibraryParamRepository.queryLibraryParamByDev(devNum);
    }

    @Transactional
    @Override
    public Integer saveBusLibraryParam(BusLibraryParamList busLibraryParamList) {
        // 得到devNum
        String devNum = busLibraryParamList.getBusLibraryParamList().get(0).getDevNum();
        // 1.根据dev_num删除bus_library_param
        Example example_BusLibraryParam = new Example(BusLibraryParam.class);
        Example.Criteria criteria_BusLibraryParam = example_BusLibraryParam.createCriteria();
        criteria_BusLibraryParam.andEqualTo("devNum", devNum);
        busLibraryParamRepository.deleteByExample(example_BusLibraryParam);
        // 2.根据dev_num删除bus_param_library
        Example example_BusParamLibrary = new Example(BusParamLibrary.class);
        Example.Criteria criteria_BusParamLibrary = example_BusParamLibrary.createCriteria();
        criteria_BusParamLibrary.andEqualTo("devNum", devNum);
        busParamLibraryRepository.deleteByExample(example_BusParamLibrary);
        // 3.根据dev_num删除bus_energy_plan
        Example example_BusEnergyPlan = new Example(BusEnergyPlan.class);
        Example.Criteria criteria_BusEnergyPlan = example_BusEnergyPlan.createCriteria();
        criteria_BusEnergyPlan.andEqualTo("devNum", devNum);
        busEnergyPlanRepository.deleteByExample(example_BusEnergyPlan);
        // 4.根据dev_num删除bus_energyplan_t
        Example example_BusEnergyplanT = new Example(BusEnergyplanT.class);
        Example.Criteria criteria_BusEnergyplanT = example_BusEnergyplanT.createCriteria();
        criteria_BusEnergyplanT.andEqualTo("devNum", devNum);
        busEnergyplanTRepository.deleteByExample(example_BusEnergyplanT);
        // 5.根据dev_num删除bus_energyplan_v
        Example example_BusEnergyplanV = new Example(BusEnergyplanV.class);
        Example.Criteria criteria_BusEnergyplanV = example_BusEnergyplanV.createCriteria();
        criteria_BusEnergyplanV.andEqualTo("devNum", devNum);
        busEnergyplanVRepository.deleteByExample(example_BusEnergyplanV);
        // 6.新增bus_library_param
        Integer result = 0;
        List<BusLibraryParam> busLibraryParams = busLibraryParamList.getBusLibraryParamList();
        for(BusLibraryParam bp : busLibraryParams){
            bp.setId(UUID.randomUUID().toString().replace("-",""));
            result = busLibraryParamRepository.insertSelective(bp);
        }
        return result;
    }


}
