package org.rcisoft.business.manage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.business.device.assets.dao.BusDeviceRepository;
import org.rcisoft.business.mainte.adaptive.dao.BusParamReferRepository;
import org.rcisoft.business.mainte.adaptive.entity.BusParamRefer;
import org.rcisoft.business.mainte.adaptive.proenum.ProEnum;
import org.rcisoft.business.manage.dao.BusDeviceMeterRepository;
import org.rcisoft.business.manage.entity.BusDeviceMeter;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.entity.BusSaving;
import org.rcisoft.business.manage.service.BusProjectAreaService;
import org.rcisoft.business.manage.service.BusProjectService;
import org.rcisoft.business.manage.service.BusSavingService;
import org.rcisoft.business.manage.service.EnergyPriceService;
import org.rcisoft.business.manage.vo.BusProjectAreaList;
import org.rcisoft.business.manage.vo.EnergyPriceList;
import org.rcisoft.business.manage.vo.EnergyPriceVo;
import org.rcisoft.business.manage.vo.ExcelParams;
import org.rcisoft.business.overview.dao.BusProjectRepository;
import org.rcisoft.business.overview.entity.BusProjectArea;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.business.totaloriginal.dao.TotalOriginalRepository;
import org.rcisoft.business.totaloriginal.dao.TotalSensorRepository;
import org.rcisoft.business.totaloriginal.entity.TotalOriginal;
import org.rcisoft.business.totaloriginal.entity.TotalSensor;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.manage.dao.ProjectRepository;
import org.rcisoft.core.service.RcRedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by  on 2018-5-2 16:09:30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusProjectServiceImpl implements BusProjectService {

    @Autowired
    private ProjectRepository projectRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private BusProjectAreaService busProjectAreaService;
    @Autowired
    private EnergyPriceService energyPriceService;
    @Autowired
    private BusSavingService busSavingService;
    @Autowired
    private RcRedisService rcRedisService;
    @Autowired
    private BusParamReferRepository busParamReferRepository;
    @Autowired
    private TotalOriginalRepository totalOriginalRepository;
    @Autowired
    private TotalSensorRepository totalSensorRepository;
    @Autowired
    private BusDeviceMeterRepository busDeviceMeterRepository;
    @Autowired
    private BusDeviceRepository busDeviceRepository;

    private Integer saveRefer(String proId) {
        Integer result = 0;
        BusParamRefer busParamRefer1 = new BusParamRefer(UUID.randomUUID().toString().replaceAll("-", ""), ProEnum.gswd.toString(), null, null, "3", proId, "供水温度");
        BusParamRefer busParamRefer2 = new BusParamRefer(UUID.randomUUID().toString().replaceAll("-", ""), ProEnum.hswd.toString(), null, null, "3", proId, "回水温度");
        BusParamRefer busParamRefer3 = new BusParamRefer(UUID.randomUUID().toString().replaceAll("-", ""), ProEnum.sll.toString(), null, null, "3", proId, "水流量");
        BusParamRefer busParamRefer4 = new BusParamRefer(UUID.randomUUID().toString().replaceAll("-", ""), ProEnum.sensor.toString(), null, null, "3", proId, "传感器名称");
        result += busParamReferRepository.insert(busParamRefer1);
        result += busParamReferRepository.insert(busParamRefer2);
        result += busParamReferRepository.insert(busParamRefer3);
        result += busParamReferRepository.insert(busParamRefer4);
        return result;
    }

    /**
    * 保存 busProject
    * @param busProject
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusProject busProject){
        Integer line = 1;
        String message = "";
        busProject.setProId(busProject.getJwnum());
        busProject.setReceiveData(0);
        busProject.setSysNm("暖通空调");
        busProject.setOnline("1");
        busProject.setCreateTime(new Date());
        BusProject bus = new BusProject();
        bus.setProId(busProject.getProId());
//        bus.setSysNm("暖通空调");
//        bus.setOnline("1");
        if(findByParam(bus)!=null && findByParam(bus).size()>0){
            line = 0;
            message = "网关编号重复，项目新增失败";
            return new PersistModel(line,message);
        }
        line = projectRepository.insertSelective(busProject);
        //新增项目成功后，添加项目能耗标准、24小时的能耗价格
        if(line>0){
            // redis添加新的内容
//            rcRedisService.setList("proIds" + busProject.getProId(), "0");
            // refer表新增4条记录，用于前台展示的参数，不可删除
            this.saveRefer(busProject.getProId());
            //能耗标准
            BusProjectAreaList busProjectAreaList = new BusProjectAreaList();
            List<BusProjectArea> list = new ArrayList<BusProjectArea>();
            BusProjectArea a1 = new BusProjectArea();
            a1.setId(UUID.randomUUID().toString().replace("-",""));
            a1.setProId(busProject.getJwnum());
            a1.setType("1");
            list.add(a1);
            BusProjectArea a2 = new BusProjectArea();
            a2.setId(UUID.randomUUID().toString().replace("-",""));
            a2.setProId(busProject.getJwnum());
            a2.setType("2");
            list.add(a2);
            BusProjectArea a3 = new BusProjectArea();
            a3.setId(UUID.randomUUID().toString().replace("-",""));
            a3.setProId(busProject.getJwnum());
            a3.setType("3");
            list.add(a3);
            busProjectAreaList.setBusProjectAreaList(list);
            PersistModel areaP =  busProjectAreaService.save(busProjectAreaList);
            //能耗价格
            EnergyPriceList energyPriceList = new EnergyPriceList();
            List<EnergyPriceVo> plist = new ArrayList<>();
            for(int i=0;i<24;i++){
                EnergyPriceVo ep_water = new EnergyPriceVo();
                ep_water.setId(UUID.randomUUID().toString().replace("-",""));
                ep_water.setProId(busProject.getJwnum());
                ep_water.setPriceHour(String.valueOf(i));
                ep_water.setPriceTp("1");
                EnergyPriceVo ep_power = new EnergyPriceVo();
                ep_power.setId(UUID.randomUUID().toString().replace("-",""));
                ep_power.setProId(busProject.getJwnum());
                ep_power.setPriceHour(String.valueOf(i));
                ep_power.setPriceTp("2");
                EnergyPriceVo ep_gas = new EnergyPriceVo();
                ep_gas.setId(UUID.randomUUID().toString().replace("-",""));
                ep_gas.setProId(busProject.getJwnum());
                ep_gas.setPriceHour(String.valueOf(i));
                ep_gas.setPriceTp("3");
                plist.add(ep_water);
                plist.add(ep_power);
                plist.add(ep_gas);
            }
            energyPriceList.setEnergyPriceList(plist);
            PersistModel priceP =  energyPriceService.save(energyPriceList);
            if(!areaP.isSuccessBySinglePersist()){
                return areaP;
            }
            if(!priceP.isSuccessBySinglePersist()){
                return priceP;
            }
        }
        return new PersistModel(line);
    }


    /**
    * 修改 busProject
    * @param busProject
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusProject busProject){
        int line = 0;
        String message = "";
        Example example = new Example(BusProject.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId", busProject.getProId());
        if(busProject.getProId()!=null && !busProject.getProId().equals("")){
            line = projectRepository.updateByExampleSelective(busProject,example);
        }else{
            message = "ID为空，更新失败";
        }
        //redis更新接收数据状态
//        if (busProject.getReceiveData() != null)
//            rcRedisService.set("proId-" + busProject.getProId(), String.valueOf(busProject.getReceiveData()));
        return new PersistModel(line,message);
    }

    @Transactional
    @Override
    public PersistModel delete(BusProject busProject) {
        int line = 0;
        String message = "";
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",busProject.getProId());
        if(busProject.getProId()!=null && !busProject.getProId().equals("")){
            //项目下没有关联的设备，可以删除
            /*if(projectRepository.queryDeivceByProject(map)==null || projectRepository.queryDeivceByProject(map).size()<1){

            }else{
                message = "项目下有关联设备，无法删除";
            }*/
            Example example = new Example(BusProject.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("proId", busProject.getProId());
            line = projectRepository.deleteByExample(example);
            // 删除redis中对应的proId
            rcRedisService.removeList("proIds", busProject.getProId());
        }else{
            message = "ID为空，删除失败";
        }
        if(line>0){
            //删除项目标准信息
            PersistModel areaP = busProjectAreaService.delete(busProject.getProId());
            //删除能耗价格信息
            PersistModel priceP = energyPriceService.delete(busProject.getProId());
            if(!areaP.isSuccessBySinglePersist()){
                return areaP;
            }
            if(!priceP.isSuccessBySinglePersist()){
                return priceP;
            }
        }
        return new PersistModel(line,message);
    }

    /**
    * 根据参数查询 busProject
    * @return
    */
    public List<Map<String,Object>> findByParam(BusProject bus){
        return projectRepository.queryBusProjects(bus);
    }

    @Override
    public BusProject findByProId(String proId) {
        BusProject busProject = new BusProject();
        busProject.setProId(proId);
        return projectRepository.selectOne(busProject);
    }

    /**
    * 分页查询 busProject
    * @param busProject
    * @return
    */
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusProject> paginationUtility,
        BusProject busProject){
        List<Map<String, Object>> list = projectRepository.queryBusProjects(busProject);
        // 将null改成""，避免前台直接显示成"null"
        list.forEach(map -> {
            String saveContent = (String) map.get("SAVE_CONTENT");
            String saveCost = (String) map.get("SAVE_COST");
            String saveEstimate = (String) map.get("SAVE_ESTIMATE");
            String saveMethod = (String) map.get("SAVE_METHOD");
            String saveShare = (String) map.get("SAVE_SHARE");
            if (StringUtils.isEmpty(saveContent) || StringUtils.equals(saveContent, "null")) map.put("SAVE_CONTENT", "");
            if (StringUtils.isEmpty(saveCost) || StringUtils.equals(saveCost, "null")) map.put("SAVE_COST", "");
            if (StringUtils.isEmpty(saveEstimate) || StringUtils.equals(saveEstimate, "null")) map.put("SAVE_ESTIMATE", "");
            if (StringUtils.isEmpty(saveMethod) || StringUtils.equals(saveMethod, "null")) map.put("SAVE_METHOD", "");
            if (StringUtils.isEmpty(saveShare) || StringUtils.equals(saveShare, "null")) map.put("SAVE_SHARE", "");
        });
        return list;
    }

    @Override
    public List<Map<String, Object>> queryBusProjectByUser(Map<String, Object> map) {
        return projectRepository.queryBusProjectByUser(map);
    }

    @Override
    public Integer receiveData(String proId, Integer receiveData) {
        //更改的对象
        BusProject busProject = new BusProject();
        busProject.setReceiveData(receiveData);
        //条件
        Example example = new Example(BusProject.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId", proId);
        int result = projectRepository.updateByExampleSelective(busProject, example);
        if (result > 0) {
            // 更新redis
            if (receiveData == 0)
                rcRedisService.removeList("proIds", proId);
            else
                rcRedisService.setList("proIds", proId);
        }
//            rcRedisService.set("proId-" + proId, String.valueOf(receiveData));
        return result;
    }

    @Override
    public void downloadExcel(HttpServletResponse response, ExcelParams excelParams) {
        String start = excelParams.getStart() + " 00:00:00";
        String end = excelParams.getEnd() + " 23:59:59";
        String code = excelParams.getCode();
        String proId = excelParams.getProId();
        Set<String> set = new HashSet<>();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<Map<String, String>> excelList = new ArrayList<>();
        Integer type = excelParams.getType();
        // 1. 根据type判断，1：查询表具，2：查询传感器，3：设备
        if (type == 1 || type == 3) {
            // 表具或设备名称集合
            Map<String, String> nameMap = this.getNm(type, proId);
            Example example = new Example(TotalOriginal.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("proId", proId);
            criteria.andBetween("tm", start, end);
            if (excelParams.getType() == 1) {
                criteria.andIsNotNull("waterV");
                criteria.andIsNotNull("elecV");
                criteria.andIsNotNull("gasV");
            } else if (excelParams.getType() == 3) {
                criteria.andIsNull("waterV");
                criteria.andIsNull("elecV");
                criteria.andIsNull("gasV");
            }
            example.setOrderByClause("DEV_NUM asc, TM asc");
            List<TotalOriginal> originalList = totalOriginalRepository.selectByExample(example);
            //解析到 excelList 中
            originalList.forEach(totalOriginal -> {
                String devNum = totalOriginal.getDevNum();
                // 根据devNum获得devNm
                String devNm = nameMap.get(devNum);
                String originalStr = totalOriginal.getOriginalStr();
                JSONObject jsonObject = JSON.parseObject(originalStr);
                String value = jsonObject.get(code) == null ? "" : jsonObject.get(code).toString();
                String time = sdf.format(totalOriginal.getTm());
                Map<String, String> map = new HashMap<>();
                map.put("time", time);
                map.put("value", value);
                map.put("nm", devNm);
                set.add(devNm);
                excelList.add(map);
            });
        } else if (type == 2) {
            Example example = new Example(TotalSensor.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("proId", proId);
            criteria.andBetween("tm", start, end);
            List<TotalSensor> sensorList = totalSensorRepository.selectByExample(example);
            sensorList.forEach(totalSensor -> {
                String sensorJson = totalSensor.getSensorJson();
                JSONObject jsonObject = JSON.parseObject(sensorJson);
                String value = jsonObject.get(code) == null ? "" : jsonObject.get(code).toString();
                String time = sdf.format(totalSensor.getTm());
                Map<String, String> map = new HashMap<>();
                map.put("time", time);
                map.put("value", value);
                excelList.add(map);
            });
        }
        // 2. 生成excel
        HSSFWorkbook workbook = new HSSFWorkbook();
        if (excelList.size() == 0) {
            HSSFSheet sheet = workbook.createSheet("sheet1");
            HSSFRow row = sheet.createRow(0);
            row.createCell(0, CellType.STRING).setCellValue("没有数据");
        } else {
            if (type == 1 || type == 3) {//表具
                set.forEach(s -> {
                    HSSFSheet sheet = workbook.createSheet(s);
                    // 第一行：时间，参数
                    HSSFRow row_1 = sheet.createRow(0);
                    row_1.createCell(0, CellType.STRING).setCellValue("时间");
                    row_1.createCell(1, CellType.STRING).setCellValue("参数：" + code);
                    // 循环list放入数据
                    for (int i = 0, k = 0; i < excelList.size(); i++) {
                        Map<String, String> param = excelList.get(i);
                        //dev_num 不一样，继续下一条循环
                        if (!StringUtils.equals(s, param.get("nm"))) {
                            k++;
                            continue;
                        }
                        HSSFRow row = sheet.createRow(i - k + 1);
                        row.createCell(0, CellType.STRING).setCellValue(param.get("time"));//时间
                        row.createCell(1, CellType.STRING).setCellValue(param.get("value"));//参数
                    }
                });
            } else {//传感器
                HSSFSheet sheet = workbook.createSheet("传感器");
                // 第一行：时间，参数
                HSSFRow row_1 = sheet.createRow(0);
                row_1.createCell(0, CellType.STRING).setCellValue("时间");
                row_1.createCell(1, CellType.STRING).setCellValue("参数：" + code);
                // 循环list放入数据
                for (int i = 0; i < excelList.size(); i++) {
                    Map<String, String> param = excelList.get(i);
                    HSSFRow row = sheet.createRow(i + 1);
                    row.createCell(0, CellType.STRING).setCellValue(param.get("time"));//时间
                    row.createCell(1, CellType.STRING).setCellValue(param.get("value"));//参数
                }
            }
        }
        // 3. 下载
        try(OutputStream outputStream = response.getOutputStream()) {
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=param.xls");
            workbook.write(outputStream);
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @Override
    public Integer checkData(String proId) {
        Integer result = 1;
        String count = rcRedisService.get(proId + "-count");
        // 如果等于空，认为是正确的值
        if (StringUtils.isNotEmpty(count)) {
            // 计数超过5，认为数据没有正确的接收
            if (Integer.parseInt(count) > 5) {
                result = 0;
            }
        }
        return result;
    }

    /**
     * 根据id，type获取表具或设备的名称
     * @param type 1：表具，2：设备
     * @param proId
     * @return
     */
    private Map<String, String> getNm(Integer type, String proId) {
        Map<String, String> resultMap = new HashMap<>();
        List<Map<String, Object>> list = null;
        if (type == 1) {
            list = busDeviceMeterRepository.queryNm(proId);
        } else if (type == 3) {
            list = busDeviceRepository.queryNm(proId);
        }
        list.forEach(map -> {
            resultMap.put(map.get("devNum").toString(), map.get("devNm").toString());
        });
        return resultMap;
    }


}
