package org.rcisoft.business.manage.service.impl;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.rcisoft.business.manage.service.EnergyEmissionPlanService;
import org.rcisoft.business.overview.dao.EnergyEmissionPlanRepository;
import org.rcisoft.business.overview.entity.EnergyEmissionPlan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ResourceUtils;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by JiChao on 2018/5/31.
 */
@Service
public class EnergyEmissionPlanServiceImpl implements EnergyEmissionPlanService {

    @Autowired
    private EnergyEmissionPlanRepository energyEmissionPlanRepository;

    /**
     * 解决中文名乱码
     * @param request
     * @param response
     * @param title
     * @param bytes
     * @throws UnsupportedEncodingException
     */
    private void setResponse(HttpServletRequest request, HttpServletResponse response, String title, byte[] bytes) throws UnsupportedEncodingException {
        String userAgent = request.getHeader("User-Agent");
        boolean isMSIE = userAgent != null && (userAgent.indexOf("MSIE") > -1 || userAgent.indexOf("like Gecko") > -1);
        if (isMSIE) {
            title = URLEncoder.encode(title, "UTF8");
        } else {
            title = new String(title.getBytes("UTF-8"), "ISO-8859-1");
        }
        response.setContentType("application/x-msdownload");
        response.setContentLength(bytes.length);
        response.setHeader("Content-Disposition", "attachment;filename=" + title + ".xls");
    }

    @Override
    public void downloadExcel(HttpServletRequest request, HttpServletResponse response) {
        try(OutputStream outputStream = response.getOutputStream()) {
            File file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "excel/碳排放量.xls");
            //转成字节
            byte[] bytes = FileUtils.readFileToByteArray(file);
            //设置下载头
            this.setResponse(request, response, "碳排放量", bytes);
            outputStream.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Transactional
    @Override
    public Integer uploadExcel(MultipartFile file, String proId) {
        HSSFWorkbook workbook = null;
        //需要保存的结果集
        List<EnergyEmissionPlan> resultList = new ArrayList<>();
        //结果
        int result = 0;
        try(InputStream is = file.getInputStream()) {
            workbook = new HSSFWorkbook(is);
            //读取sheet
            HSSFSheet sheet = workbook.getSheetAt(0);
            //从第二行开始读取数据
            for (int i = 1; i < sheet.getLastRowNum() + 1; i++) {
                HSSFRow row = sheet.getRow(i);
                if (row != null) {
                    // 如果第一列为空，认为已经结束，跳出循环
                    if (StringUtils.isEmpty(this.getValue(row.getCell(0)))) break;
                    // 从第二列开始，读取12个月份
                    for (int j = 1; j < 13; j++) {
                        // 获得单元格的值
                        String emission = this.getValue(row.getCell(j));
                        if (StringUtils.isNotEmpty(emission)) {
                            // id，proId，月，日，值
                            EnergyEmissionPlan e = new EnergyEmissionPlan(null, proId, j, i, new BigDecimal(emission));
                            resultList.add(e);
                        }
                    }
                }
            }
            // 保存数据
            if (resultList.size() > 0) {
                //先删除之前的
                energyEmissionPlanRepository.delete(new EnergyEmissionPlan(null, proId, null, null, null));
                //增加
                result = energyEmissionPlanRepository.batchSave(resultList);
            }
        }catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    @Override
    public Integer checkExcel(String proId) {
        Example example = new Example(EnergyEmissionPlan.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId", proId);
        return energyEmissionPlanRepository.selectCountByExample(example);
    }

    /**
     * 读取cell的内容
     * @param cell
     * @return
     */
    @SuppressWarnings("deprecation")
    private String getValue(HSSFCell cell) {
        if (cell == null) return null;
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_BOOLEAN:
                return cell.getBooleanCellValue() ? "TRUE" : "FALSE";
            case Cell.CELL_TYPE_FORMULA:
                return cell.getCellFormula();
            case Cell.CELL_TYPE_NUMERIC:
                cell.setCellType(Cell.CELL_TYPE_STRING);
                return cell.getStringCellValue();
            case Cell.CELL_TYPE_STRING:
                return cell.getStringCellValue();
            default:
                return null;
        }
    }
}
