package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.rcisoft.business.manage.dao.TopologyNodeRepository;
import org.rcisoft.business.manage.entity.TopologyNode;
import org.rcisoft.business.manage.service.TopologyNodeService;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-6-5 9:01:15.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TopologyNodeServiceImpl implements TopologyNodeService {

    @Autowired
    private TopologyNodeRepository topologyNodeRepository;
    @Value("${filepath.topologynode}")
    private String filepath;


    /**
    * 保存 topologyNode
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(MultipartFile file, String proId){
        int line = 1;
        String fileName = file.getOriginalFilename();
        String suffixName = fileName.substring(fileName.lastIndexOf("."));
        String id = UUID.randomUUID().toString().replace("-","");
        //文件路径
        String savePath = filepath + id + suffixName;
        File saveFile = new File(savePath);
        if(saveFile.exists()){
            saveFile.delete();
        }
        try{
            FileUtils.copyInputStreamToFile(file.getInputStream(), saveFile);
            TopologyNode tn = new TopologyNode();
            tn.setId(id);
            tn.setUrl(id + suffixName);
            tn.setProId(proId);
            topologyNodeRepository.insertSelective(tn);
            line = 1;
        }catch(Exception e){
            e.printStackTrace();
            line = 0;
        }
        return new PersistModel(line);
    }

    /**
    * 删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(String id){
        int line = 0;
        String message = "";
        if(id!=null && !id.equals("")){
            Example example = new Example(TopologyNode.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("id",id);
            line = topologyNodeRepository.deleteByExample(example);
        }else{
            message = "ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    @Override
    public List<Map<String, Object>> queryTopologyNodes(String proId) {
        return topologyNodeRepository.queryTopologyNodes(proId);
    }


}
