package org.rcisoft.business.system.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.device.assets.dao.BusDeviceRepository;
import org.rcisoft.business.device.assets.entity.BusDevice;
import org.rcisoft.business.mainte.adaptive.dao.BusParamReferRepository;
import org.rcisoft.business.mainte.adaptive.dao.BusTemperatureRepository;
import org.rcisoft.business.mainte.adaptive.dao.TotalSensorRepository;
import org.rcisoft.business.mainte.adaptive.entity.BusParamRefer;
import org.rcisoft.business.mainte.adaptive.entity.BusTemperature;
import org.rcisoft.business.mainte.adaptive.entity.TotalSensor;
import org.rcisoft.business.mainte.adaptive.proenum.ProEnum;
import org.rcisoft.business.mainte.adaptive.service.AdaptiveService;
import org.rcisoft.business.mainte.adaptive.vo.Params;
import org.rcisoft.business.manage.entity.BusFixedParam;
import org.rcisoft.business.manage.service.BusFixedParamService;
import org.rcisoft.business.overview.dao.BusProjectRepository;
import org.rcisoft.business.overview.dao.EnergyCountMRepository;
import org.rcisoft.business.overview.entity.BusProject;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.business.system.dao.DeviceParamRepository;
import org.rcisoft.business.system.entity.DeviceParam;
import org.rcisoft.business.system.service.SystemService;
import org.rcisoft.business.system.vo.ClimateStatistics;
import org.rcisoft.business.system.vo.ParamStatus;
import org.rcisoft.business.system.vo.ParamStatusList;
import org.rcisoft.core.service.RcRedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * Created by JiChao on 2018/5/2.
 */
@Service
public class SystemServiceImpl implements SystemService {
    @Autowired
    private BusParamReferRepository busParamReferRepository;
    @Autowired
    private TotalSensorRepository totalSensorRepository;
    @Autowired
    private BusTemperatureRepository busTemperatureRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private EnergyCountMRepository energyCountMRepository;
    @Autowired
    private AdaptiveService adaptiveServiceImpl;
    @Autowired
    private BusDeviceRepository busDeviceRepository;
    @Autowired
    private DeviceParamRepository deviceParamRepository;
    @Autowired
    private RcRedisService redisServiceImpl;
    @Autowired
    private BusFixedParamService busFixedParamServiceImpl;

    private String getCode(String proId) {
        BusProject b = new BusProject();
        b.setProId(proId);
        BusProject busProject = busProjectRepository.selectOne(b);
        return busProject.getCode();
    }

    @Override
    public ClimateStatistics statClimate(String proId) {
        Calendar cal = Calendar.getInstance();
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        //当前时间
        int nowHour = cal.get(Calendar.HOUR_OF_DAY);
        Params params = new Params(proId, time, this.getCode(proId));
        //查询当天气温
        List<BusTemperature> busTemperaturesList = busTemperatureRepository.queryBusTemperatureList(params);
        List<Object> outsideList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        busTemperaturesList.forEach(busTemperature -> {
            cal.setTime(busTemperature.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            if (hour > nowHour) return;
            outsideList.set(hour, busTemperature.getTemperature());
        });
        //查询 供水温度，回水温度 code
        String[] code_array = new String[2];
        List<BusParamRefer> busParamRefer = busParamReferRepository.queryOtherParam(params);
        busParamRefer.forEach(b -> {
            if(StringUtils.equals(b.getOwnParam(), ProEnum.gswd.toString())) code_array[0] = b.getOtherParam();
            if(StringUtils.equals(b.getOwnParam(), ProEnum.hswd.toString())) code_array[1] = b.getOtherParam();
        });
        //查询供回水温度
        List<TotalSensor> totalSensorList = totalSensorRepository.queryTotalSensorList(params);
        List<Object> gswdList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        List<Object> hswdList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        totalSensorList.forEach(totalSensor -> {
            cal.setTime(totalSensor.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            if (hour > nowHour) return;
            //得到json对象
            JSONObject json = JSONObject.parseObject(totalSensor.getSensorJson());
            //供水温度
            BigDecimal gswd = (BigDecimal) json.get(code_array[0]);
            //回水温度
            BigDecimal hswd = (BigDecimal) json.get(code_array[1]);
            gswdList.set(hour, gswd);
            hswdList.set(hour, hswd);
        });
        return new ClimateStatistics(outsideList, gswdList, hswdList);
    }

    @Override
    public List<Object> energyConsume(String proId) {
        Calendar cal = Calendar.getInstance();
//        cal.set(2018, 3, 28);
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setTime(time);
        List<EnergyCountMVo> energyList = energyCountMRepository.energyConsume(e);
        List<Object> energyResultList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        energyList.forEach(energyCountMVo -> {
            int hour = Integer.parseInt(energyCountMVo.getHour());
            BigDecimal energy = energyCountMVo.getEnergy();
            energyResultList.set(hour, energy);
        });
        return energyResultList;
    }

    @Override
    public List<Object> burdenHour(String proId) {
        Calendar cal = Calendar.getInstance();
//        cal.set(2018, 3, 28);
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        Params params = new Params(proId, time, this.getCode(proId));
        return adaptiveServiceImpl.buildingList(params);
    }

    @Override
    public List<BusDevice> selectErrorandRuntime(String proId) {
        List<BusDevice> list = busDeviceRepository.selectErrorandRuntime(proId);
        list.forEach(busDevice -> {
            BigDecimal runTm = busDevice.getRunTm();
            runTm = runTm == null ? new BigDecimal(0) : runTm;
            Integer errorNum = busDevice.getErrorNum();
            errorNum = errorNum == null ? 0 : errorNum;
            busDevice.setRunTm(runTm);
            busDevice.setErrorNum(errorNum);
        });
        return list;
    }

    @Override
    public Map<String, Object> selectDeviceParamList(String devNum) {
        // 表具的键
        String devCode = "";
        // 电量的键
        String devParam = "";
        // proId
        String proId = devNum.split("_")[0];
        // 根据devNum查询
        Map<String, String> deviceMap = deviceParamRepository.selectPower(devNum);
        if (deviceMap != null) {
            devCode = deviceMap.get("devCode");
            devParam = deviceMap.get("devParam");
//            proId = deviceMap.get("proId");
        }
        List<BusFixedParam> fixedList = busFixedParamServiceImpl.queryBusFixedParamList(devNum, proId);
        List<DeviceParam> list = deviceParamRepository.selectDeviceParamList(devNum, proId);
        // 如果不为空，需要将devCode截取一下
        if (StringUtils.isNotEmpty(devCode) && StringUtils.isNotEmpty(devParam) && StringUtils.isNotEmpty(proId))
            devCode = devCode.split(proId + "_")[1];
        Map<String, Object> result = new HashMap<>();
        result.put("data", list);
        result.put("fixData", fixedList);
        result.put("devCode", devCode);
        result.put("devParam", devParam);
        return result;
    }

    @Override
    public Map<String, String> selectSensorCode(String proId, String devNum) {
        String sensor = deviceParamRepository.selectSensorCode(proId);
        String power = deviceParamRepository.selectMetNum(proId, devNum);
        if (StringUtils.isNotEmpty(power)) {
            power = power.split(proId + "_")[1];
        }
        // 返回值
        Map<String, String> resultMap = new HashMap<>();
        resultMap.put("sensor", sensor);
        resultMap.put("power", power);
        return resultMap;
    }

    @Override
    public String selectDataByCache(String proId) {
//        String newData = redisServiceImpl.get(proId);
//        String oldData = redisServiceImpl.get(proId + "-old");
//        return "{\"newData\":" + newData + ",\"oldData\":" + oldData + "}";
        return redisServiceImpl.get(proId);
    }

    @Override
    public List<ParamStatus> selectDeviceAll(String proId) {
        // 根据proId得到redis中的数据
        String data = redisServiceImpl.get(proId);
        // 得到json数据
        JSONObject json = JSON.parseObject(data);
        List<ParamStatus> list = deviceParamRepository.selectDeviceAll(proId);
        list.forEach(paramStatus -> {
            // 设备来源
            String devNum = paramStatus.getDevNum();
            // 参数
            String param = paramStatus.getParam();
            // 如果devNum不是空，截取字符串
            if (StringUtils.isNotEmpty(devNum)) {
                devNum = devNum.split(proId + "_")[1];
            }
            paramStatus.setDevNum(devNum);
            // 默认都设为掉线状态
//            paramStatus.setStatus(2);
            // 默认值为0
            paramStatus.setValue(0);
            // 获得数据
            if (json != null) {
                JSONObject jsonObject = (JSONObject) json.get(devNum);
                if (jsonObject != null) {
                    JSONObject valueObject = (JSONObject) jsonObject.get("REG_VAL");
                    if (valueObject != null) {
                        Object value = valueObject.get(param);
                        if (value != null) {
                            paramStatus.setValue(value);
                        }
                    }
                }
            }
        });
        return list;
    }

    @Override
    public List<ParamStatus> selectDeviceStatus(ParamStatusList list) {
        // 得到proId
        String proId = "";
        List<ParamStatus> paramStatusList = list.getList();
        if (paramStatusList.size() > 0) {
            proId = paramStatusList.get(0).getProId();
        }
        if (!proId.equals("")) {
            // 根据proId得到redis中的数据
            String newData = redisServiceImpl.get(proId);
            JSONObject newJson = JSON.parseObject(newData);
            if (newJson != null) {
                // 循环，获取两个数据做对比
                for (ParamStatus paramStatus : paramStatusList) {
                    String devNum = paramStatus.getDevNum();
                    String param = paramStatus.getParam();
                    if (StringUtils.isNotEmpty(devNum) && StringUtils.isNotEmpty(param)) {
                        // 新数据
                        JSONObject newObj = (JSONObject) newJson.get(devNum);
                        if (newObj != null) {
                        // 赋值status
                            Integer status = (Integer) newObj.get("status");
                            // status=1，在线状态，继续判断
                            // status=0，掉线状态，前台直接变成掉线状态
                            paramStatus.setStatus(status);
                            JSONObject newVal = (JSONObject) newObj.get("REG_VAL");
                            if (newVal != null) {
                                Object o = newVal.get(param);
                                if (o != null) {
                                    // 赋值value
                                    paramStatus.setValue(o);
                                }
                            }
                        }
                    }
                }
            }
        }
        return paramStatusList;
    }
}
