package org.rcisoft.business.system.user.dao;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:422018/5/2
 */
@Repository
public interface UserRepository extends BaseMapper<SysUser>{

    /**  根据用户名称查找用户信息
     * @param userName
     * @return
     */
    List<SysUser> listByUserName(@Param("userName") String userName);

    /** 根据用户ID查看用户信息
     * @param userId
     * @return
     */
    List<Map<String,Object>> getUserInfo(@Param("userId") String userId);

    /**
     * 根据openid查找用户信息，查看是否绑定
     * @param openId
     * @return
     */
    @Select("<script>select * from sys_user where openid = #{openId}</script>")
    List<Map<String,Object>> getUserByOpenId(@Param("openId") String openId);


    /**
     * 查询所有用户
     * @return
     */
    @Select("<script>select a.USER_ID,a.USER_NM,a.USER_NICK_NAME,a.USER_TP " +
            "<if test=\"stable != null\">,b.*</if> " +
            "from sys_user a " +
            "<if test=\"stable != null\">left join ${stable} b on a.USER_ID = b.USER_ID </if> " +
            "where 1=1" +
            "<if test = \"userTp=='1'.toString()\"> and (user_tp = '1' or user_tp = '2')</if>" +
            "<if test = \"userTp=='2'.toString()\"> and user_tp = '3'</if>" +
            "<if test = \"userTp=='3'.toString()\"> and user_tp = '4'</if>" +
            "<if test = \"userTp=='5'.toString()\"> and user_tp = '5'</if>" +
            " order by a.create_date desc</script>")
    List<Map<String,Object>> listUserAll(@Param("stable") String stable,@Param("userTp") String userTp);

}
