package org.rcisoft.business.system.user.service;

import org.apache.ibatis.annotations.Param;
import org.rcisoft.business.system.user.entity.*;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;

import java.util.List;
import java.util.Map;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:422018/5/2
 */
public interface UserService {


     /**
      * 登录
      * @param username
      * @param password
      * @return
      */
     String login(String username, String password);

     /**  添加系统用户
      * @param sysUser
      * @param sysAdmins
      * @param sysOwner
      * @param sysPrincipal
      * @return
      */
     int saveUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner,SysPrincipal sysPrincipal);


     /**  修改系统用户
      * @param sysUser
      * @param sysAdmins
      * @param sysOwner
      * @param sysPrincipal
      * @return
      */
     int updateUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner,SysPrincipal sysPrincipal);

     /**
      * 更新用户openId
      * @param userId 用户主键
      * @param openId 微信openId
      * @return
      */
     int updateUserOpenid(String userId,String openId);

     /** 根据userId删除用户
      * @param userId
      * @return
      */
     String deleteUser(String userId);

     /** 根据userId查找用户
      * @param userId
      * @return
      */
     Object getUserByUserId(String userId);

     /** 根据userId查出可以看到的菜单
      * @param userId
      * @return
      */
     List<Map<String,Object>> listMenuBuUserId(String userId);


     /** 维护角色可以看到的菜单
      * @param relRoleMenu
      * @return
      */
     int updateRoleMenu(RelRoleMenu relRoleMenu);

     /** 查看某个用户的信息
      * @return
      */
     List<Map<String,Object>> getUserInfo(String userId);


     /** 修改密码
      * @param newPassWord
      * @param oldPassWord
      * @return
      */
     PersistModel updatePassWord(String newPassWord, String oldPassWord, String userId);

     /** 重置密码
      * @param userId
      * @return
      */
     int resetPassWord(String userId, String needUserId);


     List<Map<String,Object>> listMenuAll();

     List<Map<String,Object>> listRoleAll();

     List<Map<String,Object>> listTeamAll();

     List<Map<String,Object>> listOwnerAll();

     List<Map<String,Object>> listUserAllByPagination(PageUtil<SysUser> paginationUtility,String userTp);

     Map<String, Object> loginWechat(SysUser user);

     Map<String, Object> getUserByOpenId(@Param("openId") String openId);

    /**
     * 根据用户id查询用户
     * @param ids
     * @return
     */
     List<SysUser> getUsersByIds(String ids);
}
