package org.rcisoft.business.system.user.service.impl;

import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.device.assets.entity.BusDevice;
import org.rcisoft.business.evaluate.team.dao.BusTeamRepository;
import org.rcisoft.business.manage.dao.SysUserProjectRepository;
import org.rcisoft.business.system.user.dao.*;
import org.rcisoft.business.system.user.entity.*;
import org.rcisoft.business.system.user.service.UserService;
import org.rcisoft.common.constants.SysRoleConstant;
import org.rcisoft.common.constants.UserPassWordConstant;
import org.rcisoft.common.constants.UserTpConstant;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.JwtUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.*;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:432018/5/2
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class UserServiceImpl implements UserService {
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private AdminRepository adminRepository;
    @Autowired
    private OwnerRepository ownerRepository;
    @Autowired
    private PrincipalRepository principalRepository;
    @Autowired
    private RelRoleMenuRepository relRoleMenuRepository;
    @Autowired
    private RelUserRoleRepository relUserRoleRepository;
    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private RoleRepository roleRepository;
    @Autowired
    private BusTeamRepository busTeamRepository;
    @Autowired
    private SysUserProjectRepository sysUserProjectRepository;

    @Override
    public String login(String username, String password) {
        UsernamePasswordAuthenticationToken upToken = new UsernamePasswordAuthenticationToken(username, password);
        //进入到 UserDetailsService(UserDetailServiceImpl)  loadUserByUsername 方法
        final Authentication authentication = authenticationManager.authenticate(upToken);
        UserDetails userDetails = (UserDetails)authentication.getPrincipal();
        final String token = JwtUtil.generateToken(userDetails);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        return token;
    }

    @Override
    public int saveUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner, SysPrincipal sysPrincipal) {
        int flag = 0;

        String userNm = sysUser.getUserNm();
        String userTp = sysUser.getUserTp();

        // 验重
        Example example = new Example(SysUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("userNm", userNm);
        List<SysUser> flagSysUsers = userRepository.selectByExample(example);
        if (flagSysUsers.size() <= 0){
            String userId = IdGen.uuid();
            sysUser.setUserPwd(passwordEncoder.encode(UserPassWordConstant.PASSWORD));
            sysUser.setUserId(userId);
            sysUser.setCreateDate(new Date());
            try {
                if (UserTpConstant.TP_ADMIN.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysAdmins.setAdminId(IdGen.uuid());
                    sysAdmins.setUserId(userId);
                    adminRepository.insertSelective(sysAdmins);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_ADMIN_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                if (UserTpConstant.TP_OWNER.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysOwner.setOwnId(IdGen.uuid());
                    sysOwner.setUserId(userId);
                    ownerRepository.insertSelective(sysOwner);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_OWNER_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysPrincipal.setPriId(IdGen.uuid());
                    sysPrincipal.setUserId(userId);
                    principalRepository.insertSelective(sysPrincipal);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_PRINCIPAL_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                if(UserTpConstant.TP_INSPECTPERSON.equals(userTp)){
                    flag = userRepository.insertSelective(sysUser);
                }
                return flag;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return flag;
    }

    @Override
    public int updateUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner, SysPrincipal sysPrincipal) {
        int flag = 0;
        String userId = sysUser.getUserId();
        String userTp = sysUser.getUserTp();
        try {
            if (UserTpConstant.TP_ADMIN.equals(userTp) || UserTpConstant.TP_SUPER_ADMIN.equals(userTp)) {
                SysAdmins sysAdminsByUserId = adminRepository.getSysAdminsByUserId(userId);
                if (null != sysAdminsByUserId){
                    sysAdmins.setAdminId(sysAdminsByUserId.getAdminId());
                    adminRepository.updateByPrimaryKeySelective(sysAdmins);
                    flag = 1;
                }
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {
                SysOwner sysOwnerByUserId = ownerRepository.getSysOwnerByUserId(userId);
                if (null != sysOwnerByUserId) {
                    sysOwner.setOwnId(sysOwnerByUserId.getOwnId());
                    ownerRepository.updateByPrimaryKeySelective(sysOwner);
                    flag = 1;
                }
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                SysPrincipal sysPrincipalByUserId = principalRepository.getSysPrincipalByUserId(userId);
                if (null != sysPrincipalByUserId) {
                    sysPrincipal.setPriId(sysPrincipalByUserId.getPriId());
                    principalRepository.updateByPrimaryKeySelective(sysPrincipal);
                    flag = 1;
                }
            }
            if (UserTpConstant.TP_INSPECTPERSON.equals(userTp)) {
                flag = userRepository.updateByPrimaryKeySelective(sysUser);
            }
            return flag;
        } catch (Exception e) {
            e.printStackTrace();
        }
       return flag;
    }

    @Override
    public int updateUserOpenid(String userId, String openId) {
        SysUser user = new SysUser();
        user.setUserId(userId);
        user.setOpenid(openId);
        Example example = new Example(SysUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("userId",userId);
        int line = userRepository.updateByExampleSelective(user,example);
        return line;
    }

    @Override
    public String deleteUser(String userId) {
        String result = "删除失败";
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);

        try {
            SysUser selectUser = userRepository.selectByPrimaryKey(sysUser);

            userRepository.deleteByPrimaryKey(sysUser);

            RelUserRole relUserRole = new RelUserRole();
            relUserRole.setUserId(userId);
            relUserRoleRepository.deleteByPrimaryKey(relUserRole);

            String userTp = selectUser.getUserTp();
            if (UserTpConstant.TP_ADMIN.equals(userTp)) {

                adminRepository.deleteSysAdminByUserId(userId);

                result = "删除成功";
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {

                ownerRepository.deleteSysOwnerByUserId(userId);

                result = "删除成功";
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {

                principalRepository.deleteSysPrincipalByUserId(userId);

                result = "删除成功";
            }
            return result;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return "删除失败";
    }

    @Override
    public Object getUserByUserId(String userId) {

        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        try {
            SysUser selectUser = userRepository.selectByPrimaryKey(sysUser);

            String userTp = selectUser.getUserTp();
            if (UserTpConstant.TP_ADMIN.equals(userTp)) {
               return adminRepository.getSysAdminsByUserId(userId);
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {
                return ownerRepository.getSysOwnerByUserId(userId);
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                return principalRepository.getSysPrincipalByUserId(userId);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public List<Map<String, Object>> listMenuBuUserId(String userId) {
        List<Map<String, Object>> list =  relUserRoleRepository.listMenuBuUserId(userId);
        return handleMenu(list);
    }

    @Override
    public int updateRoleMenu(RelRoleMenu relRoleMenu) {
        int flag = relRoleMenuRepository.updateByPrimaryKeySelective(relRoleMenu);
        return flag;
    }

    @Override
    public List<Map<String, Object>> getUserInfo(String userId) {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        // 1.查出用户名等信息
        SysUser sysUser_param = new SysUser();
        sysUser_param.setUserId(userId);
        SysUser sysUser = userRepository.selectOne(sysUser_param);
        // 放入结果
        map.put("USER_ID", sysUser.getUserId());
        map.put("USER_NM", sysUser.getUserNm());
        map.put("USER_TP", sysUser.getUserTp());
        // 2.根据user_tp查询对应的用户信息
        String userTp = sysUser.getUserTp();
        if (userTp.equals("1") || userTp.equals("2")) {// 管理员
            SysAdmins sysAdmins_param = new SysAdmins();
            sysAdmins_param.setUserId(userId);
            SysAdmins sysAdmins = adminRepository.selectOne(sysAdmins_param);
            map.put("ADMIN_ID", sysAdmins.getAdminId());
            map.put("ADMIN_NM", sysAdmins.getAdminNm());
            map.put("TELEPHONE", sysAdmins.getTelephone());
            map.put("EMAIL", sysAdmins.getEmail());
            map.put("WECHAT", sysAdmins.getWechat());
        } else if (userTp.equals("3")) {// 业主
            SysOwner sysOwner_param = new SysOwner();
            sysOwner_param.setUserId(userId);
            SysOwner sysOwner = ownerRepository.selectOne(sysOwner_param);
            map.put("OWN_ID", sysOwner.getOwnId());
            map.put("OWN_NM", sysOwner.getOwnNm());
            map.put("TELEPHONE", sysOwner.getTelephone());
            map.put("EMAIL", sysOwner.getEmail());
            map.put("WECHAT", sysOwner.getWechat());
        } else if (userTp.equals("4")) {// 负责人
            SysPrincipal sysPrincipal_param = new SysPrincipal();
            sysPrincipal_param.setUserId(userId);
            SysPrincipal sysPrincipal = principalRepository.selectOne(sysPrincipal_param);
            map.put("PRI_ID", sysPrincipal.getPriId());
            map.put("PRI_NM", sysPrincipal.getPriNm());
            map.put("JOB", sysPrincipal.getJob());
            map.put("JOBTITLE", sysPrincipal.getJobtitle());
            map.put("BUSHOUR", sysPrincipal.getBushour());
            map.put("HONOR", sysPrincipal.getHonor());
        }
        list.add(map);
        return list;
    }

    @Override
    public PersistModel updatePassWord(String newPassWord, String oldPassWord, String userId) {
        int flag = 0;
        String message = "";
        SysUser sysUser = userRepository.selectByPrimaryKey(userId);
        String encodedPassword = sysUser.getUserPwd();
        boolean matches = passwordEncoder.matches(oldPassWord, encodedPassword);
        if (matches){
            sysUser.setUserPwd(passwordEncoder.encode(newPassWord));
            int i = userRepository.updateByPrimaryKeySelective(sysUser);
            flag = i;
        }else{
            message = "原密码错误，修改失败";
        }
        return new PersistModel(flag,message);
    }

    @Override
    public int resetPassWord(String userId, String needUserId) {
        int flag = 0;
        SysUser sysUser = userRepository.selectByPrimaryKey(userId);
        if (UserTpConstant.TP_SUPER_ADMIN.equals(sysUser.getUserTp()) || UserTpConstant.TP_ADMIN.equals(sysUser.getUserTp())){
            SysUser needSysUser = userRepository.selectByPrimaryKey(needUserId);
            needSysUser.setUserPwd(passwordEncoder.encode(UserPassWordConstant.PASSWORD));
            userRepository.updateByPrimaryKeySelective(needSysUser);
            flag = 1;
        }
        return flag;
    }

    @Override
    public List<Map<String, Object>> listMenuAll() {
        List<Map<String, Object>> list = relUserRoleRepository.listMenuAll();
        return handleMenu(list);
    }

    @Override
    public List<Map<String, Object>> listRoleAll() {
        //
        List<Map<String, Object>> list = roleRepository.listRoleAll();
        for(Map<String, Object> mp : list){
            String menuIds = mp.get("MENU_ID").toString();
            String[] menuArr = menuIds.split(",");
            List<String> menuList = new ArrayList<>();
            for(String s : menuArr){menuList.add(s);}
            mp.put("MENULIST",handleMenu(roleRepository.queryMenuByIds(menuList)));
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> listTeamAll() {
        return busTeamRepository.listTeamAll();
    }

    @Override
    public List<Map<String, Object>> listOwnerAll() {
        return ownerRepository.listOwnerAll();
    }

    @Override
    public List<Map<String, Object>> listUserAllByPagination(PageUtil<SysUser> paginationUtility,String userTp) {
        String table  = null;
        if(userTp!=null && userTp.equals("1")){table = "sys_admins";}
        if(userTp!=null && userTp.equals("2")){table = "sys_owner";}
        if(userTp!=null && userTp.equals("3")){table = "sys_principal";}
        List<Map<String, Object>> list =  userRepository.listUserAll(table,userTp);
        for(Map<String, Object> mp : list){
            mp.put("REL_PROJECT",sysUserProjectRepository.querySysUserPro(mp.get("USER_ID").toString()));
        }
        return list;
    }

    @Override
    public Map<String, Object> loginWechat(SysUser user) {
        Map<String, Object> result = new HashMap<>();
        int line = 0;
        String message = "";
        Example example = new Example(SysUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("userNm", user.getUserNm());
        criteria.andEqualTo("userTp", 5);
        List<SysUser> list = userRepository.selectByExample(example);
//        List<SysUser> list = userRepository.listByUserName(user.getUserNm());
        if(list!=null && list.size()==1){
            SysUser su = list.get(0);
            boolean flag = passwordEncoder.matches(user.getUserPwd(),su.getUserPwd());
            if(flag){//登录成功后，关联微信openId
                line = updateUserOpenid(su.getUserId(),user.getOpenid());
                if(line<1){
                    message = "授权失败";
                }else{
                    result.put("userNickName", su.getUserNickName());
                    message = "授权成功";
                }
            }else{
                message = "登录失败";
            }
        }else{
            message = "登录异常";
        }
        result.put("line", line);
        result.put("message", message);
        return result;
    }

    @Override
    public Map<String, Object> getUserByOpenId(String openId) {
        Map<String, Object> result = new HashMap<>();
        List<Map<String,Object>> list = userRepository.getUserByOpenId(openId);
        if(list!=null && list.size()>0){
            result.put("count", 1);
            result.put("userNickName", list.get(0).get("USER_NICK_NAME"));
        } else {
            result.put("count", 0);
        }
        return result;
    }

    @Override
    public List<SysUser> getUsersByIds(String ids) {
        if (StringUtils.isNotEmpty(ids)) {
            String[] split = ids.split(",");
            List<String> list = Arrays.asList(split);
            Example example = new Example(SysUser.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andIn("userId", list);
            return userRepository.selectByExample(example);
        } else {
            return null;
        }
    }

    public List<Map<String, Object>> handleMenu(List<Map<String, Object>> menuList){
        List<Map<String, Object>> result = new ArrayList<>();
        for(Map<String, Object> tmap : menuList){
            if(tmap.get("MENU_PID")==null){
                Map<String,Object> map = new HashMap<>();
                map.put("id",tmap.get("MENU_ID"));
                map.put("label",tmap.get("MENU_NM"));
                map.put("url",tmap.get("MENU_URL"));
                result.add(map);
            }
        }
        for(Map<String, Object> rmap : result){
            String pid = rmap.get("id").toString();
            List<Map<String, Object>> childList = new ArrayList<>();
            for(Map<String, Object> tmap : menuList){
                if(tmap.get("MENU_PID")!=null && tmap.get("MENU_PID").toString().equals(pid)){
                    Map<String, Object> m = new HashMap<>();
                    m.put("id",tmap.get("MENU_ID"));
                    m.put("url",tmap.get("MENU_URL"));
                    m.put("label",tmap.get("MENU_NM"));
                    childList.add(m);
                }
            }
            rmap.put("children",childList);
        }
        return result;
    }
}
